extern void lwfftw_r2c_1d(SIGNAL in, SIGNAL out, unsigned fftw_flags);
extern void C_FftwConvolByPart(char **argv);
static void FftwConvolByPart(SIGNAL in, SIGNAL filter, int size_window, SIGNAL out) {
  /* filter should be an YSIG, M = filter->size should be < in->size,
     filter->x0 and filter->dx should be such that i0 = -x0/dx is an integer
     pointing to the X=0 index of the filter. 
     0 <= i0 < M (hence x0 should be < 0)
     m_pos = M-1-i0 is the filter size with > X
     m_neg = i0 is the filter size with < X
     M = m_pos+m_neg+1

     size_window should be > M+1
  */
  SIGNAL ft_in, filter_perio, ft_filter;

  LWFLOAT Re, Im;
  
  int M = filter->size;
  int i0 = (int) (0.5-filter->x0/filter->dx);
  int m_pos = M-1-i0;
  int m_neg = i0;
  int i,j,k, ir, ii, num_windows, step_window, size_window_fft;

  Printf("Filter size: %d\tm+: %d\tm-: %d\n",M,m_pos,m_neg);

  /* Some consistency checking */
  if ( (i0 < 0 ) || (i0 >= M) ) {
    Errorf("FftwConvolByPart(): i0 (=%d) should be in range [0,%d[ !!",i0,M);
  }
  if ( size_window <= M+1 ) {
    Errorf("FftwConvolByPart(): Window size (=%d) should > filter size + 1 (=%d) !!",size_window, M+1);
  }
  /* ************************* */
  
  /* FT of in */
  ft_in = TNewSignal();
  step_window = size_window-(M-1);
  num_windows = lwfftw_multiple_r2c_1d_old(in, ft_in, size_window, step_window, FFTW_ESTIMATE);
  Printf("Num windows: %d\tSize window: %d\tStep window: %d\tSize FT in: %d\n", num_windows, size_window, step_window, ft_in->size);
  
  /* Construct filter perio*/
  filter_perio = TNewSignal();
  SizeSignal(filter_perio, size_window, YSIG);
  for(i=0;i <= m_pos;i++) {filter_perio->Y[i] = filter->Y[i0+i];}
  for(;i < size_window-m_neg;i++) {filter_perio->Y[i] = 0.0;}
  for(;i < size_window;i++) {filter_perio->Y[i] = filter->Y[i-(size_window-m_neg)];}
         
  /* FT of filter */
  ft_filter = TNewSignal();
  lwfftw_r2c_1d(filter_perio, ft_filter, FFTW_ESTIMATE);
  /* Prod FTs */
  size_window_fft = size_window/2+1;
  Printf("Size ft_filter: %d\tSize window fft: %d\n",ft_filter->size, size_window_fft);
  for(k =0; k<num_windows; k++) {
    for(i=0; i<size_window_fft;i++) {
      ir = 2*i;
      ii = 2*i+1;
      Re = ft_in->Y[2*k*size_window_fft+ir];
      Im = ft_in->Y[2*k*size_window_fft+ii];
      ft_in->Y[2*k*size_window_fft+ir] = Re*ft_filter->Y[ir]-Im*ft_filter->Y[ii];
      ft_in->Y[2*k*size_window_fft+ii] = Re*ft_filter->Y[ii]+Im*ft_filter->Y[ir];
    }
  }

  /* IFFT */
  lwfftw_multiple_c2r_1d(ft_in, out, size_window, num_windows, FFTW_ESTIMATE);
  
  /* Reconstruct the convolution in place */
  /* We dont touch coeff corresponding to the first window */
  j = size_window-m_neg; /* first index computed from second window */
  for(k=1;  k<num_windows; k++) {
    for(i=m_pos;i<size_window-m_neg;i++) {
      out->Y[j] = out->Y[k*size_window+i];
      j++;
    }
  }
  out->lastp = j-1;
}

void C_FftwConvolByPart(char **argv) {
  SIGNAL in, filter, out;
  int size_window;
  LWFLOAT time;

  argv = ParseArgv(argv,tSIGNALI,&in,tSIGNALI,&filter,tINT, &size_window,tSIGNAL,&out,0);

  time = MyTime();
  FftwConvolByPart(in, filter, size_window, out);
  SetResultFloat(MyTime()-time);
}

static int lwfftw_multiple_r2c_1d_old(SIGNAL in, SIGNAL out, int size_window, int step_window, unsigned fftw_flags) {
  lw_fftw_plan p;
  int howmany = 1+(in->size-size_window)/step_window;
  int size_window_out = size_window/2+1;

  /* out will contain the output complex results: out->size = 2*(in->size/2+1) */
  SizeSignal(out,howmany*2*size_window_out,YSIG);

  p = LW_FFTW_NAMES(plan_many_dft_r2c)(1, &size_window, howmany,
				       in->Y, NULL, 1, step_window,
				       (lw_fftw_complex *) out->Y, NULL, 1, size_window_out,
				       fftw_flags);
  if (p == NULL) {Errorf("Problem while making plan !!");}

  LW_FFTW_NAMES(execute)(p);
  
  LW_FFTW_NAMES(destroy_plan)(p);

  return howmany;
}
