/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

#ifndef POLLUX_H
#define POLLUX_H

#include <stdio.h>
#include <string.h>

#include "long_string.h"

#ifndef NOT_USED_WITH_LW
#include "double.h"
#else
#define FLOAT float
#include <float.h>
#define FLOAT_MAX FLT_MAX
#endif /* NOT_USED_WITH_LW */


#define POLSTRSIZE 256 /* IT MUST BE GREATER THAN 51 */
#define POLSIZELIST 60
#define POLBUFFERSIZE 50000000

/* FOR pollux_tri_b.c */
/*  #define POLSIZESIGNALLIST 30 */
/*  #define POLSIZESTRINGLIST 3000 */

/* ALL THE ERROR VALUES MUST NEGATIVE */
#define POLMEMERROR -5
#define POLOVERFLOW -4
#define POLFILEIOERROR -3
#define POLFORMATERROR -2
#define POLERROR -1
#define POLOK 0

/* FOR pol_read.c */
#define POLFILLFULL 1
#define POLFILLMOTIF 2
#define POLFILLPOS 3

#define POLNO 0
#define POLYES 1

#define LMAX 2500

/* TYPE POLBUFFER*/

typedef struct PolluxBuffer{
  char buffer[POLBUFFERSIZE];
  int size;
}PolluxBuffer;


/* SOME EXTERNAL DECLARATIONS */
extern void Printf(char *format,...);

/*  IN pol_struct.c */
extern void PolInitList( char polList[][POLSTRSIZE]);
extern void PolDisplayError( int polSta, char *procedureName);
extern int IsOfCodingType( const char *type);
extern int IsOfStringList(const char *string,
			  const char stringList[][POLSTRSIZE], int *pPos);
extern int PolStrCpy(char *string,const char *value);
extern int PolStrCat(char *string,const char *value);
extern char *PolStrLower(char *string);
extern int PolLineToBuffer(PolluxBuffer *polbuffer, char *line);
extern int PolFieldValueToLine( char *line, char *field, char *value);
extern int *AdnSigToInt(FLOAT *signal, int size);
extern int DerIntSig( int *intSignal, int size);
extern char *RemoveLeadBlank( char *string );
extern void RemoveTrailBlank( char *string );
extern int PolFileToSpeciesNum( const char *filePath,char *speciesPath,
				char *num);

/* IN pol_write.c */
extern int PolWrite(const char *fileName,
		    char fieldList[][POLSTRSIZE],
		    char valueList[][POLSTRSIZE]);
extern int PolRemoveField(const char *fileName,
			  char fieldList[][POLSTRSIZE]);
extern int PolAddField(const char *fileName,
		       char fieldList[][POLSTRSIZE]);

/* IN pol_read.c */
extern int PolReadAll(FILE *fpFile,char fieldList[][POLSTRSIZE],
		      char valueList[][POLSTRSIZE], PolluxBuffer *seqBuffer);
extern int PolRead(const char *fileName,
		   char fieldList[][POLSTRSIZE],
		   char valueList[][POLSTRSIZE]);
extern int PolBufferToSignal(PolluxBuffer *polbuffer,FLOAT *signal,
			     char *codage);
extern int PolCodPolyNucBufferToSignal(PolluxBuffer *seqBuffer,FLOAT *signal,
				       char **codList,int polyLength);
extern int PolCodPolyNucStringToSignal(char *seq, int sizeb, FLOAT *signal,
				       char **codList, int polyLength);
extern int PolDistantPolyNucBufferToSignal(PolluxBuffer *seqBuffer,FLOAT *signal,
					   char **codList1,char **codList2,
					   int polyLength,int *distTab,int distNum,
					   int flagFill);

/* IN pol_seq.c */
/* WARNING: _PolSeqBufferToFileBuffer(): Doesn't take care of putting the "=SEQUENCE\n" before the sequence */
extern int _PolSeqBufferToFileBuffer(PolluxBuffer *fileBuffer,PolluxBuffer *seqBuffer);
extern int PolSeqBufferToFile(const char *fileName,PolluxBuffer *seqBuffer);
extern int PolBufferToFile(const char *fileName,PolluxBuffer *fileBuffer);
extern int PolSequenceToBuffer( const char *fileName, PolluxBuffer *seqBuffer);
extern int PolOppStrandBuffer(PolluxBuffer *seqBuffer);
extern int PolOppStrandSequence(const char *fileName);
extern int PolSeqToBuffer(FILE *fp,PolluxBuffer *seqBuffer);
extern int PolSeqToLongString(FILE *fp,LONGSTRING ls);

/* UNDERSAMPLES THE SEQUENCE IN FILE fileName CONSIDERING THAT THE FIRST
   COMPLETE CODON IS IN POSITION phase AND THAT THE RESULTING SEQUENCE
   SHOULD BE A SEQUENCE OF BASES IN POSITION phaseSubSeq IN ITS CODON */
extern int PolSubSampleBuffer(PolluxBuffer *seqBuffer,int phase,
			      int phaseSubSeq);
extern int PolSubSampleSequence(const char *fileNameIn,const char *fileNameOut,
				int phaseSubSeq);

/* IN pol_sim.c */
extern FLOAT PolSimilitude(const char *polbufferA,const char *polbufferB,
			   int n);

/* IN pol_extract.c */
extern int PolExtract(const char *archiveName,const char *speciesPath);

/* Functions in random_seq.c */
extern int ran_seq(PolluxBuffer *ranSeqBuffer,int size,FLOAT a,FLOAT t,FLOAT g);

#endif /* POLLUX_H */
