/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

#include "helix_3d.h"


#ifndef NOT_USED_WITH_LW

extern void *Malloc(size_t size);
extern void Free(void * ptr);
extern void Printf(char *format,...);
#define malloc Malloc
#define free Free
#define printf Printf

#endif /* NOT_USED_WITH_LW */


/* PRODUIT MATRICIELLE 3*3 */
static void prod_mat(double m1[3][3], double m2[3][3],
		     double m[3][3])
{
  int i,j,k;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      m[i][j] = 0;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      for(k=0;k<3;k++)
	m[i][j] += m1[i][k]*m2[k][j];
}

/* COPIE DE MATRICE */
static void mat_copy(double in[3][3],double out[3][3])
{
  int i,j;  
  
  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      out[i][j] = in[i][j];
}
/* MATRICE DE ROTATION AUTOUR DE X */
static void mat_rot_x(double m[3][3],double angle)
{
  m[2][2] = m[1][1] = cos(angle);
  m[2][1] = sin(angle);
  m[1][2] = -m[2][1];
  m[0][0] = 1.;
  m[0][1] = m[0][2] = m[1][0] = m[2][0] = 0.;
}
/* MATRICE DE ROTATION AUTOUR DE Y */
static void mat_rot_y(double m[3][3],double angle)
{
  m[2][2] = m[0][0] = cos(angle);
  m[0][2] = sin(angle);
  m[2][0] = -m[0][2];
  m[1][1] = 1.;
  m[1][0] = m[2][1] = m[0][1] = m[1][2] = 0.;
}
/* MATRICE DE ROTATION AUTOUR DE Z */
static void mat_rot_z(double m[3][3],double angle)
{
  m[1][1] = m[0][0] = cos(angle);
  m[0][1] = sin(angle);
  m[1][0] = -m[0][1];
  m[2][2] = 1.;
  m[2][0] = m[2][1] = m[0][2] = m[1][2] = 0.;
}


static void add_trans(double m[3][3],double twist,double roll,double tilt)
     /*
       Mat(B2 dans B0) = Mat(B1 dans B0)*Mat(B2 dans B1)
       Donc il faut multiplier a droite pour additionner une matrice
       */
{
  static double t[3][3];
  static double temp[3][3];

  /* LE TWIST */
  mat_rot_z(t,twist);
  prod_mat(m,t,temp);

  /* LE ROLL */
  mat_rot_x(t,roll);
  prod_mat(temp,t,m);

  /* LE TILT */
  mat_rot_y(t,tilt);
  prod_mat(m,t,temp);

  mat_copy(temp,m);

}

static void sous_trans(double m[3][3],double twist,double roll,double tilt)
     /*
       Mat(B2 dans B0) = Mat(B1 dans B0)*Mat(B2 dans B1)
       Donc il faut multiplier a gauche pour soustraire une matrice
       */
{
  static double t[3][3];
  static double temp[3][3];

  /* LE TWIST */
  mat_rot_z(t,-twist);
  prod_mat(t,m,temp);

  /* LE ROLL */
  mat_rot_x(t,-roll);
  prod_mat(t,temp,m);

  /* LE TILT */
  mat_rot_y(t,-tilt);
  prod_mat(t,m,temp);

  mat_copy(temp,m);

}



static double cos_angle(double m[3][3])
{
  return m[2][2];
}
  
#define INDEXA 0
#define INDEXT 1
#define INDEXG 2
#define INDEXC 3
#define INDEXO 4

static double the_twist_mat[5][5][5];
static double the_roll_mat[5][5][5];
static double the_tilt_mat[5][5][5];
static int flag_are_init_the_mat = 0;

char index_2_nuc(int i)
{
  switch(i)
    {
    case INDEXA:
      return 'a';
    case INDEXT:
      return 't';
    case INDEXG:
      return 'g';
    case INDEXC:
      return 'c';
    default:
      break;
    }
  return '\0';
}

int nuc_2_index(char nuc)
{
  switch(nuc)
      {
      case 'a':
      case 'A':
	return INDEXA;
      case 't':
      case 'T':
	return INDEXT;
      case 'g':
      case 'G':
	return INDEXG;
      case 'c':
      case 'C':
	return INDEXC;
      default:
	break;
      }
  return INDEXO;
}

static int read_the_mat(char *filename,double the_mat[5][5][5])
{
  int i,j,k;
  FILE *fp;

  fp = fopen(filename,"r");
  if(fp ==NULL)
    return -1;

  for(k=0;k<4;k++)
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
	{
	  if( fscanf(fp,"%lf",&the_mat[i][j][k]) != 1)
	    {
	      fclose(fp);
	      return -1; 
	    }
	}

  fclose(fp);

  return 0;
}

static int init_the_roll_mat(char *filename)
{
  int i,j,k;

  if(read_the_mat(filename,the_roll_mat) != 0)
    return -1;
  
  for(i=0;i<5;i++)
    for(j=0;j<5;j++)
      {
	the_roll_mat[i][j][4] = 0.;
	the_roll_mat[i][4][j] = 0.;
	the_roll_mat[4][i][j] = 0.;
      }

  for(k=0;k<4;k++)
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
	the_roll_mat[i][j][k] *= M_PI/180.;

  return 0;
}

static int init_the_tilt_mat(char *filename)
{
  int i,j,k;

  if(read_the_mat(filename,the_tilt_mat) != 0)
    return -1;
  
  for(i=0;i<5;i++)
    for(j=0;j<5;j++)
      {
	the_tilt_mat[i][j][4] = 0.;
	the_tilt_mat[i][4][j] = 0.;
	the_tilt_mat[4][i][j] = 0.;
      }

  for(k=0;k<4;k++)
    for(i=0;i<4;i++)
      for(j=0;j<4;j++)
	the_tilt_mat[i][j][k] *= M_PI/180.;

  return 0;
}

static int init_the_twist_mat(void)
{
  int i,j,k;
  
  for(k=0;k<5;k++)
    for(i=0;i<5;i++)
      for(j=0;j<5;j++)
	the_twist_mat[i][j][k] = 34.3*M_PI/180.;

  return 0;
}


static void get_tw_r_ti(char *seq,int index,double *ptw,double *pr,double *pti)
{
  int k[3];
  int i;
  
  for(i=0;i<3;i++)
    k[i] = nuc_2_index(seq[index+i]);
  
  *ptw = the_twist_mat[k[0]][k[1]][k[2]];
  *pr = the_roll_mat[k[0]][k[1]][k[2]];
  *pti = the_tilt_mat[k[0]][k[1]][k[2]];
}

static void init_mat_identity(double m[3][3])
{
  int i,j;

  for(i=0;i<3;i++)
    for(j=0;j<3;j++)
      m[i][j] = 0.;

  for(i=0;i<3;i++)
    m[i][i] = 1.;

}

static char roll_file_default[] = "/user31/audit/MySofts/BEND/src/roll.dat";
static char *roll_file = roll_file_default;
int change_roll_file(const char *filename)
{
  /* if filename == NULL we want to use the default file */
  if(filename == NULL)
    {
      /* roll_file == roll_file_default there is nothing to do */
      if(roll_file == roll_file_default)
	return 0;
      else
	{
	  free(roll_file);
	  roll_file = roll_file_default;
	  flag_are_init_the_mat = 0;
	  return 0;
	}
    } 
  else
    {
      /* if filename[] ==  roll_file[] there is nothing to do */
      if(strcmp(filename,roll_file) == 0)
	return 0;
      else
	{
	  if(roll_file != roll_file_default)
	    free(roll_file);
	  
	  roll_file = (char *) malloc((strlen(filename)+1)*sizeof(char));

	  if(roll_file == NULL)
	    {
	      roll_file = roll_file_default;
	      flag_are_init_the_mat = 0;
	      return -1;
	    }

	  strcpy(roll_file,filename);
	  flag_are_init_the_mat = 0;
	  return 0;
	}
    }

  return 0;
}
      
static char tilt_file_default[] = "/user31/audit/MySofts/BEND/src/tilt.dat";
static char *tilt_file = tilt_file_default;
int change_tilt_file(const char *filename)
{

  /* if filename == NULL we want to use the default file */
  if(filename == NULL)
    {
      /* tilt_file == tilt_file_default there is nothing to do */
      if(tilt_file == tilt_file_default)
	return 0;
      else
	{
	  free(tilt_file);
	  tilt_file = tilt_file_default;
	  flag_are_init_the_mat = 0;
	  return 0;
	}
    } 
  else
    {
      /* if filename[] ==  tilt_file[] there is nothing to do */
      if(strcmp(filename,tilt_file) == 0)
	return 0;
      else
	{
	  if(tilt_file != tilt_file_default)
	    free(tilt_file);
	  
	  tilt_file = (char *) malloc((strlen(filename)+1)*sizeof(char));

	  if(tilt_file == NULL)
	    {
	      tilt_file = tilt_file_default;
	      flag_are_init_the_mat = 0;
	      return -1;
	    }

	  strcpy(tilt_file,filename);
	  flag_are_init_the_mat = 0;
	  return 0;
	}
    }

  return 0;
}

int compute_all_angle(char *seq,FLOAT *Y,int size,int size_window)
{
  double m[3][3];
  double tw,r,ti;
  int i;

  /* INITIALISATION */
  if(flag_are_init_the_mat != 1)
    {
      init_the_twist_mat();
      printf("roll file: %s\n",roll_file);
      if( init_the_roll_mat(roll_file) != 0)
	return -1;
      printf("tilt file: %s\n",roll_file);
      if( init_the_tilt_mat(tilt_file) != 0)
	return -1;
      flag_are_init_the_mat = 1;
    }
  /*  Printf("1");Flush(); */
  init_mat_identity(m);
  for(i=0;i<size_window;i++)
    {
      /*      Printf("o");Flush(); */
      get_tw_r_ti(seq,i,&tw,&r,&ti);
      add_trans(m,tw,r,ti);
    }
  /*  Printf("*");Flush(); */
  for(i=0;i<=size_window/2;i++)
    Y[i]=0.;

  /* PREMIER POINT */
  Y[size_window/2+1] = cos_angle(m);
  
  /* THE BOUCLE */
  for(i=1;i<=size-size_window-2;i++)
    {
      /*      Printf("*");Flush();*/
      get_tw_r_ti(seq,i+size_window-1,&tw,&r,&ti);
      add_trans(m,tw,r,ti);

      get_tw_r_ti(seq,i-1,&tw,&r,&ti);
      sous_trans(m,tw,r,ti);

      Y[i+size_window/2+1] = cos_angle(m);

    }

  /* LES DERNIERS POINTS */
  for(i=size-size_window+size_window/2;i<size;i++)
    Y[i] = 0.;

  return 0;
}

int compute_all(char *seq,FLOAT *X,FLOAT *Y,FLOAT *Z,int size)
{
  double m[3][3];
  double tw,r,ti;
  int i;

  /* INITIALISATION */
  if(flag_are_init_the_mat != 1)
    {
      init_the_twist_mat();
      printf("roll file: %s\n",roll_file);
      if( init_the_roll_mat(roll_file) != 0)
	return -1;
      printf("tilt file: %s\n",tilt_file);
      if( init_the_tilt_mat(tilt_file) != 0)
	return -1;
      flag_are_init_the_mat =1;
    }
  /* PREMIER POINT */
  X[0] = 0.;
  Y[0] = 0.;
  Z[0] = 1.;

  init_mat_identity(m);
  for(i=0;i<size-2;i++)
    {
      get_tw_r_ti(seq,i,&tw,&r,&ti);
      add_trans(m,tw,r,ti);
      X[i+1] = m[0][2];
      Y[i+1] = m[1][2];
      Z[i+1] = m[2][2];
    }

  /* LE DERNIER POINT */
  X[size-1] = 0.;
  Y[size-1] = 0.;
  Z[size-1] = 0.;

  return 0;
}

int compute_courb_table(FLOAT tab[4][4][4])
{
  double m[3][3];
  double tw,r,ti;
  int i,j,k;
  double x,y,z;

  /* INITIALISATION */
  if(flag_are_init_the_mat != 1)
    {
      init_the_twist_mat();
      printf("roll file: %s\n",roll_file);
      if( init_the_roll_mat(roll_file) != 0)
	return -1;
      printf("tilt file: %s\n",tilt_file);
      if( init_the_tilt_mat(tilt_file) != 0)
	return -1;
      flag_are_init_the_mat =1;
    }

  for(i=0;i<4;i++)
    {
      for(j=0;j<4;j++)
	{
	  for(k=0;k<4;k++)
	    {
	      /* LET'S GET THE ANGLES */
	      tw = the_twist_mat[i][j][k];
	      r = the_roll_mat[i][j][k];
	      ti = the_tilt_mat[i][j][k];

	      /* LET'S COMPUTE THE CURBATURE (|d2M/d2s|) */
	      init_mat_identity(m);
	      add_trans(m,tw,r,ti);
	      x = m[0][2];
	      y = m[1][2];
	      z = m[2][2];
	      tab[i][j][k] = sqrt(x*x+y*y+(z-1.)*(z-1.));

	      /* Printf("%c%c%c %f\n",index_2_nuc(i),index_2_nuc(j),
		 index_2_nuc(k),tab[i][j][k]); */
	    }
	}
    }
  
  return 0;
}

