/*..........................................................................*/
/*                                                                          */
/*      Long String 0.1                                                     */
/*                                                                          */
/*      Authors: Benjamin Audit.                                            */
/*      Copyright (C) 2005.  All Right Reserved.                            */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

#include <stdlib.h>
#include <string.h>

#include "long_string.h"


#ifndef NOT_USED_WITH_LW

extern void *Malloc(size_t size);
extern void *Realloc(void *ptr, size_t size);
extern void Free(void * ptr);

/* extern void Printf(char *format,...); */

#define malloc Malloc
#define realloc Realloc
#define free Free

#endif /* NOT_USED_WITH_LW */


#define LS_RATE_SIZE_INCREASE 0.15 /* Size of long string are increased by 15% streps */
#define LS_MIN_SIZE_INCREASE 10000 /* with a minimum step size of 10000 */

/* TYPE LongString*/
struct LongString {
  char *str; /* The null terminated string*/
  size_t len; /* The current length excluding the terminating null char */
  size_t sizeMalloc; /* The size allocated using malloc or realloc*/
};


LONGSTRING LSNew(void) {
  LONGSTRING ls;
  /*   Printf("LSNew()"); */

  ls = (LONGSTRING) malloc(sizeof(struct LongString));
  if( ls == NULL ) return NULL;

  ls->str = NULL;
  ls->len = 0;
  ls->sizeMalloc = 0;

  return ls;
}

void LSDelete(LONGSTRING ls) {
  if( ls->str != NULL ) 
    free(ls->str);

  free(ls);
}

static LONGSTRING LSIncreaseSizeMalloc(LONGSTRING ls, size_t minSizeMalloc) {
  char *newStr;
  size_t newSizeMalloc;
  size_t sizeMallocIncrease = LS_RATE_SIZE_INCREASE * ls->sizeMalloc;
  
  if( sizeMallocIncrease < LS_MIN_SIZE_INCREASE ) 
    sizeMallocIncrease = LS_MIN_SIZE_INCREASE;

  newSizeMalloc = ls->sizeMalloc + sizeMallocIncrease;

  if( minSizeMalloc > newSizeMalloc ) 
    newSizeMalloc = minSizeMalloc;
  
  newStr = (char *) realloc(ls->str, sizeof(char)*newSizeMalloc);

  if(newStr == NULL) 
    return NULL;

  if(ls->str == NULL)  /* This is to insure that the first time there is allocation a terminating null byte is set*/
    newStr[0]='\0';

  ls->sizeMalloc = newSizeMalloc;
  ls->str = newStr;

  return ls;
}

void LSFreeSpace(LONGSTRING ls) {
  free(ls->str);
  ls->sizeMalloc = 0;
  ls->len =0;
}

void LSSetStringEmpty(LONGSTRING ls) {
  if( ls->sizeMalloc ==  0 )
    return;

  ls->str[0]='\0';
  ls->len = 0;
}

/* is it really useful .... ?? */
static LONGSTRING LSShrinkSizeMalloc(LONGSTRING ls) {
  char *newStr = (char *) realloc(ls->str, sizeof(char)*(ls->len+1));

  if(newStr == NULL) 
    return NULL;
  
  ls->sizeMalloc = ls->len+1;
  ls->str = newStr;

  return ls;
}

/****************************************************************************/
/* CONCATENATES A CHAR * AT THE END OF A LONGSTRING                         */
/****************************************************************************/
LONGSTRING LSConcatLine(LONGSTRING ls, char *line)
{
  size_t len =strlen(line);

  /* do we need to allocate more memory ? */
  if( len+ls->len+1 > ls->sizeMalloc ) {
    if( LSIncreaseSizeMalloc(ls,len+ls->len+1) == NULL )
      return NULL;
  }
  
  /* Consistency check, we should not have to do that */
  /*   if ( *(ls->str+ls->len) != '\0') { */
  /*     return NULL; */
  /*   } */

  strcat(ls->str+ls->len,line);
  ls->len += len;
  
  return ls;
}

/****************************************************************************/
/* Get Fonctions                                                            */
/****************************************************************************/

/* return the C string */
/* YOU SHOULD NOT MODIFY it in any way */
char *LSGetString(LONGSTRING ls) {
  return ls->str;
}

/* return the length of the string */
size_t LSStringLength(LONGSTRING ls) {
  return ls->len;
}
