/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

/****************************************************************************/
/*                                                                          */
/*               EXTRACT POLLUX FILES FROM A POLLUX ARCHIVE                 */
/*                                                                          */
/****************************************************************************/



#include "pollux.h"
#include <time.h>
#include <unistd.h>


/****************************************************************************/
/*       EXTRACT ALL POLLUX FILES FROM A POLLUX ARCHIVE                     */
/*                                                                          */
/****************************************************************************/
/* THIS FUNCTION ASSUME THAT THE DIR speciesDir and THE FILE speciesDir/info*/
/* ALREADY EXIST                                                            */
/*                                                                          */
/****************************************************************************/
int PolExtract(const char *archiveName,const char *speciesPath)
{
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  char fieldList2[POLSIZELIST][POLSTRSIZE],valueList2[POLSIZELIST][POLSTRSIZE];
  FILE *fpArchive,*fpFile;
  char c,fileName[POLSTRSIZE],line[POLSTRSIZE],temp[POLSTRSIZE];
  int iMax,polSta,flagClosed;
  time_t date = time(NULL);
  struct tm *date_local = localtime(&date);

  /* OPENING ARCHIVE FILE archiveName */
  /* WARNING: THIS MUST BE DONE BEFORE CHANGING WORK DIRECTORY */
  if((fpArchive = fopen(archiveName,"r")) == NULL)
    {
      Printf("Couldn't open file: %s",archiveName);
      return POLFILEIOERROR;
    }
    
  /* CHANGING WORK DIRECTORY TO speciesPath */
  if(chdir(speciesPath) != 0) 
    {
      Printf("Couldn't access to directory: %s",speciesPath);
      fclose(fpArchive);
      return POLFILEIOERROR;
    }

  /* WE READ IMAX FROM THE INFO FILE */
  PolInitList(fieldList);
  PolInitList(valueList);
  strcpy(fieldList[0],"IMAX");

  if((polSta = PolRead("info",fieldList,valueList)) != POLOK)
    {
      PolDisplayError(polSta,"PolRead"); 
      fclose(fpArchive);
      return polSta;
    }


  /* WE READ IMAX FROM valueList[0] TESTING FOR FORMAT ERROR */
  if(sscanf(valueList[0],"%d %s",&iMax,temp) != 1)
    {
      fclose(fpArchive);
      return POLFORMATERROR;
    }
  

  /* INITIALISATION */
  PolInitList(fieldList2);
  PolInitList(valueList2);
  strcpy(fieldList2[0],"CRPP");
  strcpy(fieldList2[1],"DATE");
  strftime(valueList2[1],POLSTRSIZE,"%m %d %y",date_local);

  /* THE FILE WILL BE TREATED LINE BY LINE */
  while(fgets(line,POLSTRSIZE,fpArchive)!=NULL)
    {
      /* FIRST WE MUST FIND THE BEGINNING OF THE FILE */
      if(sscanf(line,"%1s",&c) < 1) /* IT IS AN EMPTY LINE: GOTO NEXT LINE */
	continue;
      else if(c != '>') /* IT IS AN INTER FILE LINE: GOTO NEXT LINE */
	continue;
      else /* WE ARE AT THE BEGINNING OF A FILE LET'S EXTRACT IT */ 
	{
	  flagClosed = 0;
	  /* WE INCREMENT iMax */
	  iMax++;
	  /* WE OPEN THE NEW POLLUX FILE: IT MUSTN'T EXIST BEFORE */
	  sprintf(fileName,"%d",iMax);
	  if((fpFile = fopen(fileName,"r")) != NULL)
	    {
	      Printf("File %s already exist in %s !!!\n",fileName,speciesPath);
	      fclose(fpArchive);
	      fclose(fpFile);
	      return POLFORMATERROR;
	    }
	  if ((fpFile = fopen(fileName,"w")) ==NULL)
	    {
	      Printf("Can't open file: %s in %s\n",fileName,speciesPath);
	      fclose(fpArchive);
	      return POLFILEIOERROR;
	    }
	  /* WE COPY THE CONTENT OF THE NEW FILE */
	  while(fgets(line,POLSTRSIZE,fpArchive)!=NULL)
	    {
	      if(sscanf(line,"%1s",&c) < 1) /* IT IS AN EMPTY LINE: GOTO NEXT LINE */
		continue;
	      if(c == '<') /* WE REACH THE LAST LINE OF THE FILE */
		{
		  /* WE SET flagClosed TO 1 TO SAY THAT AN END OF FILE < WAS FOUND */
		  flagClosed =1;

		  if (fclose(fpFile) == EOF) 
		    {
		      Printf("Probleme while closing file: %s\n",fileName);
		      fclose(fpArchive);
		      return POLFILEIOERROR;
		    }
		  /* WE ADD SOME FIELDS */
		  sprintf(valueList2[0],"%d",iMax);
		  if((polSta = PolAddField(fileName,fieldList2)) != POLOK)
		    {
		      PolDisplayError(polSta,"PolAddField"); 
		      fclose(fpArchive);
		      return polSta;
		    }
		  if((polSta = PolWrite(fileName,fieldList2,valueList2)) != POLOK)
		    {
		      PolDisplayError(polSta,"PolWrite"); 
		      fclose(fpArchive);
		      return polSta;
		    }


		  /* EVERY THING WENT OK SO WE WRITE THE NEW VALUE OF IMAX TO info FILE */
		  sprintf(valueList[0],"%d",iMax);
		  if((polSta = PolWrite("info",fieldList,valueList)) != POLOK)
		    {
		      PolDisplayError(polSta,"PolWrite"); 
		      fclose(fpArchive);
		      fclose(fpFile);
		      return polSta;
		    }

		  /* WE GO BACK TO LOOK FOR THE BEGINNING OF THE NEXT FILE */
		  break;
		}

	      /* IT IS A LINE TO COPY */
	      else if(fputs(line,fpFile) == EOF)
		{
		  Printf("Probleme while writing file: %s\n",fileName);
		  fclose(fpFile);
		  fclose(fpArchive);
		  return POLFILEIOERROR;
		}
	    }
	  /* IF flagClosed != 1 IT MEANS NO CLOSING "<" HAS BEEN FOUND */
	  /* BEFORE THE END OF THE ARCHIVE FILE */
	  if(flagClosed != 1)
	    {
	      Printf("No closing < were found while making file: %s\n",fileName);
	      fclose(fpFile);
	      fclose(fpArchive);
	      return POLFORMATERROR;
	    }
	}
    }
  fclose(fpArchive);
  return POLOK;
}

