/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

/****************************************************************************/
/*                                                                          */
/*            PACKAGE TO USE pollux WHITH LastWave                          */
/*                                                                          */
/****************************************************************************/



#include "pol_package.h"
#include "helix_3d.h"
/* Some global variables to avoid limited stacksize problem */
static PolluxBuffer fileBuffer;
static PolluxBuffer seqBuffer;
static PolluxBuffer buffer;


/* A function that returns the size of a list */
static int SizeList(char **list)
{
  int size = 0;

  while (*list != NULL) 
    {
      size++;
      list++;
    }
  
  return size;
}
/****************************************************************************/
/* READ THE OPTIONNAL ARGUMENTS FOR POLLUX PROCEDURE: IT PUTS THE FIELDS    */
/* IN fieldList[i] AND THE CORRESPONDING CHARACTERE STRING IN stringList[i] */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/
int PolOptArgs( char **argv, char fieldList[][POLSTRSIZE], char stringList[][POLSTRSIZE])
{
  int polSta;
  int i,is,size;
  char **listTemp;

  argv = ParseArgv(argv,tLIST_,NULL,&listTemp,-1);

  i=0;
  while(listTemp != NULL)
    {

      if(i == POLSIZELIST)
	{
	  Printf("Too many arguments\n");
	  return POLFORMATERROR;
	}
      
      size = SizeList(listTemp);
      
      if( size == 0 )
	Errorf("PolOptArgs(): !!!!! ");
      else
	{
	  polSta = PolStrCpy(fieldList[i],listTemp[0]);
	  if(polSta != POLOK)
	    {
	      PolDisplayError(polSta,"PolStrCpy()");
	      return polSta;
	    }
	  
	  if(size >=2)
	    {
	      stringList[i][0] = '\0';
	      for(is=1;is < size;is++)
		{
		  polSta = PolStrCat(stringList[i]," ");
		  if(polSta != POLOK)
		    {
		      PolDisplayError(polSta,"PolStrCat()");
		      return polSta;
		    }

		  polSta = PolStrCat(stringList[i],listTemp[is]);
		  if(polSta != POLOK)
		    {
		      PolDisplayError(polSta,"PolStrCat()");
		      return polSta;
		    }
		}
	    }
	  else
	    strcpy(stringList[i]," ");
	}
      /*      DeleteList(listTemp); */
      argv = ParseArgv(argv,tLIST_,NULL,&listTemp,-1);
      i++;
    }
  
#ifdef DEBUGALLOC 
  for(i=0;(fieldList[i][0] != '\0') && (i < POLSIZELIST);i++)
    {
      Printf("%s,%s.\n",fieldList[i],stringList[i]);
    }
#endif

  return POLOK;
}

/****************************************************************************/
/* DISPLAY AN ERROR MESSAGE ACCORDING TO A RETURN VALUE                     */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/

void PolDisplayError(int polSta, char *procedureName)
{
  char message[POLSTRSIZE];

  switch(polSta)
    {
    case POLERROR:
      sprintf(message,"Program error in %s",procedureName);  
      break;
    case POLFORMATERROR:
      sprintf(message,"Format error in %s",procedureName);
      break;
    case POLFILEIOERROR:
      sprintf(message,"I/O error in %s",procedureName);
      break;
    case POLOVERFLOW:
      sprintf(message,"Overflow error in %s",procedureName);
      break;
    case POLMEMERROR:
      sprintf(message,"Not enough memory in %s",procedureName);
      break;
    default:
      sprintf(message,"Error in %s: unexpected return value",procedureName);
      break;
    }
  
  Printf("%s\n",message);
}

/****************************************************************************/
/*                                                */
/****************************************************************************/
/****************************************************************************/
void C_PolSubSampleSequence(char **argv)
{
  char *fileNameIn,*fileNameOut;
  int phaseSubSeq;
  int polSta;

  argv = ParseArgv(argv,tSTR,&fileNameIn,tSTR,&fileNameOut,
		   tINT,&phaseSubSeq,0);

  if( (polSta = PolSubSampleSequence(fileNameIn,fileNameOut,phaseSubSeq)) != POLOK)
    {
      PolDisplayError(polSta,"PolSubSampleSequence");
      Errorf("");
    }
}




void C_PolExtract(char **argv)
{
  char *archiveName,*speciesPath;
  int polSta;

  argv = ParseArgv(argv,tSTR,&archiveName,tSTR,&speciesPath,0);

  if((polSta = PolExtract(archiveName,speciesPath)) != POLOK)
    {
      PolDisplayError(polSta,"PolExtract");
      Errorf("");
    }
}


 
void C_PolFileToSpeciesNum( char **argv)
{
  char *filePath,*speciesPathVar,*numVar;
  char speciesPath[POLSTRSIZE],num[POLSTRSIZE];
  int polSta;

  argv = ParseArgv(argv,tSTR,&filePath,tVNAME,&speciesPathVar,tVNAME,&numVar,0);

  if( (polSta = PolFileToSpeciesNum(filePath,speciesPath,num)) != POLOK)
    {
      PolDisplayError(polSta,"PolFileToSpeciesNum");
      Errorf("");
    }

  SetStrVariable(speciesPathVar,speciesPath);
  SetStrVariable(numVar,num);

  AppendResultf("{%s %s}",speciesPath,num);

  return;
}

static int _PolReadAll(char *fileName,char *name,int flagGetSeq)
{
  char tempName[POLSTRSIZE];
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  int i,polSta;
  VALUE value;
  HASHTABLE t;
  FILE *fpFile;

  /* Can we open the file ? */
  if ( (fpFile = FOpen(fileName,"r")) == NULL)
    {
#ifdef BENDEBUG
      Printf("file %s can't be open\n",fileName);
#endif
      return -1;
    }
  
  /* Yes, Going on */

  value = ExistVariable(name,NULL);
  if (value != NULL) {
    if (GetTypeValue(value) != arrayType)
      Errorf("_PolReadAll(): Expect an &array  argument");
    t = (HASHTABLE) value;
    ClearHashTable(t);
  } else {
    t = NewHashTable(POLSIZELIST);
    SetVariable(name,(VALUE) t);
    DeleteHashTable(t);
  }
  
  
  PolInitList(fieldList);
  PolInitList(valueList);


  /* WE READ THE FIELDS AND VALUES */
  if(( polSta = PolReadAll(fpFile,fieldList,valueList,NULL)) != POLOK) 
     {
       PolDisplayError(polSta,"PolReadAll");
       Errorf("");
     }

  /* We Can now close the stream */
  FClose(fpFile);
  
  /* WE SET THE NEW VARIABLE */
  i = 0;
  while(fieldList[i][0] != '\0' && i < POLSIZELIST)
    {
      if ( (polSta =PolStrCpy(tempName,name)) != POLOK)
	{
	  PolDisplayError(polSta,"PolStrCpy");
	  Errorf("");
	}
      if ( (polSta =PolStrCat(tempName,".")) != POLOK)
	{
	  PolDisplayError(polSta,"PolStrCat");
	  Errorf("");
	}
      if ( (polSta =PolStrCat(tempName,fieldList[i])) != POLOK)
	{
	  PolDisplayError(polSta,"PolStrCat");
	  Errorf("");
	}
      
      /* Ben Modif 23/09/02 */
      /* SetStrVariable(PolStrLower(tempName),valueList[i]); */
      SetStrVariable(tempName,valueList[i]);
      /* End Modif */
      i++;
    }

  /* Ben Modif 23/09/02 */
  if(flagGetSeq != 0) {
    
    /* THE buffer IS INITIALIZE IN PolSequenceToBuffer */
    if( (polSta = PolSequenceToBuffer(fileName,&seqBuffer)) != POLOK ) {
	PolDisplayError(polSta,"PolSequenceToBuffer");
	Errorf("");
      }
    if ( (polSta =PolStrCpy(tempName,name)) != POLOK) {
      PolDisplayError(polSta,"PolStrCpy");
      Errorf("");
    }
    if ( (polSta =PolStrCat(tempName,".SEQUENCE")) != POLOK) {
      PolDisplayError(polSta,"PolStrCat");
      Errorf("");
    }
    SetStrVariable(tempName,seqBuffer.buffer);
  }
  /* End Modif */
  
  return 1;
}

/****************************************************************************/
/* READ ALL field AND value IN FILE speciesPath/num                         */
/* AND PUT RESULT IN THE HASHTABLE VARIABLE NAMED field                     */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/
void C_PolReadAll(char **argv)
{
  char *fileName,*name;
  int flagGetSeq = 0;
  char opt;

  argv = ParseArgv(argv,tSTR,&fileName,tVNAME,&name,-1);
  
  while(opt = ParseOption(&argv)) {
    switch(opt) {
    case 'S': /* SEQUENCE */
      flagGetSeq = 1;
      break;      
    default:
      ErrorOption(opt);
    }
  }
  NoMoreArgs(argv);

  SetResultf("%d",_PolReadAll(fileName,name,flagGetSeq));
}

void C_PolSpeciesReadAll(char **argv)
{
  char opt;
  char *specieName,*name;
  char fileName[POLSTRSIZE],numName[POLSTRSIZE],varName[POLSTRSIZE];
  int i,imax,polsta;
  int length,specieLength,nameLength;
  int flagGetSeq = 0;
  VALUE value;
  HASHTABLE t;
  SIGNAL signal = NULL;

  argv = ParseArgv(argv,tSTR,&specieName,tVNAME,&name,-1);

  value = ExistVariable(name,NULL);
  if (value != NULL) {
    if (GetTypeValue(value) != arrayType)
      Errorf("C_PolSpeciesReadAll(): Expect an &array as second argument");
    t = (HASHTABLE) value;
    ClearHashTable(t);
  } else {
    t = NewHashTable(1000);
    SetVariable(name,(VALUE) t);
    DeleteHashTable(t);
  }

  /* The options */
  imax = -1;
  while(opt = ParseOption(&argv))
    {
      switch(opt) 
        {
	case 'i': /* imax */
	  argv = ParseArgv(argv,tINT,&imax,-1);
	  if(imax<0) Errorf("Bad range for imax");
	  break;      
	case 's': /* signal*/
	  argv = ParseArgv(argv,tSIGNALI,&signal,-1);
	  imax = signal->size;
	  break;      
	case 'S': /* SEQUENCE */
	  flagGetSeq = 1;
	  break;      
	default:
          ErrorOption(opt);
        }
    }
  NoMoreArgs(argv);
  if(imax<0) Errorf("One of the options -i or -s is required");
  
  /* Some inits */
  if( (polsta = PolStrCpy(fileName,specieName)) != POLOK )
    {
      PolDisplayError(polsta,"PolStrCpy");
      Errorf("");
    }
  if( (polsta = PolStrCat(fileName,"/SEQ/")) != POLOK )
    {
      PolDisplayError(polsta,"PolStrCat");
      Errorf("");
    }
  specieLength = strlen(fileName);

  if( (polsta = PolStrCpy(varName,name)) != POLOK )
    {
      PolDisplayError(polsta,"PolStrCpy");
      Errorf("");
    }
  if( (polsta = PolStrCat(varName,".")) != POLOK )
    {
      PolDisplayError(polsta,"PolStrCat");
      Errorf("");
    }
  nameLength = strlen(varName);

  buffer.buffer[0] = '\0';
  buffer.size = 0;

#ifdef BENDEBUG
  Printf("%s %s\n",fileName,varName);
  Flush();
#endif

  /* Let's go */
  for(i=1;i<=imax;i++)
    {
      if(signal == NULL)
	sprintf(numName,"%d",i);      
      else
	sprintf(numName,"%.0f",signal->Y[i-1]);
      

      fileName[specieLength]='\0';
      if( (polsta = PolStrCat(fileName,numName)) != POLOK )
	{
	  PolDisplayError(polsta,"PolStrCat");
	  Errorf("");
	}

      varName[nameLength]='\0';
      if( (polsta = PolStrCat(varName,numName)) != POLOK )
	{
	  PolDisplayError(polsta,"PolStrCat");
	  Errorf("");
	}

#ifdef BENDEBUG
      Printf("%s %s %s\n",numName,fileName,varName);
      Flush();
#endif

      if(_PolReadAll(fileName,varName,flagGetSeq)!=1)
	{
	  Printf("**** file %s doesn't exist\n",fileName);
	  continue;
	}
      
      length = strlen(numName);
      numName[length] = ' ';
      numName[length+1] = '\0';
      if( (polsta = PolLineToBuffer(&buffer,numName)) != POLOK )
	{
	  PolDisplayError(polsta,"PolLineToBuffer");
	  Errorf("");
	}
      
#ifdef BENDEBUG
      Printf("%s, %s\n",numName,buffer.buffer);
      Flush();
#endif
    }
  
  varName[nameLength]='\0';
  if( (polsta = PolStrCat(varName,"list.all")) != POLOK )
    {
      PolDisplayError(polsta,"PolStrCat");
      Errorf("");
    }

  SetStrVariable(varName,buffer.buffer);
  
}

/****************************************************************************/
/* Writes all the hashelem of an array into a pollux file                   */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/
void C_PolWriteAll(char **argv)
{
  char *fileName;
  char line[POLSTRSIZE];
  int r,polSta;
  VALUE value;
  HASHTABLE t;
  VARIABLE e;

  /* LET'S INITIALIZE THE buffers */
  seqBuffer.size = 0;
  seqBuffer.buffer[0] = '\0';
  fileBuffer.size = 0;
  fileBuffer.buffer[0] = '\0';

  argv = ParseArgv(argv,tSTR,&fileName,tVAL,&value,0);
  if (GetTypeValue(value) != arrayType)
    Errorf("Expect an &array as second argument");
  t = (HASHTABLE) value;
  if (t == NULL) Errorf1("");

  /* Loop on the array elem */
  for (r = 0; r<t->nRows;r++) {
    for (e = (VARIABLE) (t->rows[r]); e != NULL; e = (VARIABLE) (e->next)) {
      value = *(GetVariablePContent(e,NULL));
      /* We expect a STRVALUE */
      if (value == NULL || GetTypeValue(value) != strType) 
	Errorf1("variable %s.%s is not of string type",t->name,e->name);
      
      /* Yes, lets put it in the fileBuffer if it is not the SEQUENCE field */
      /* otherwise let's fill the seqBuffer */
      if( strcmp(e->name,"SEQUENCE") ) {
	if( (polSta = PolFieldValueToLine(line,e->name,
					  GetStrFromStrValue((STRVALUE) value)))
	    != POLOK ) {
	  PolDisplayError(polSta,"PolFieldValueToLine");
	  Errorf("");
	}
	if( (polSta = PolLineToBuffer(&fileBuffer,line)) != POLOK ) {
	  PolDisplayError(polSta,"PolLineToBuffer");
	  Errorf("");
	}
      } else {
	if( (polSta = PolLineToBuffer(&seqBuffer,
				      GetStrFromStrValue((STRVALUE) value)))
	    != POLOK ) {
	  PolDisplayError(polSta,"PolLineToBuffer");
	  Errorf("");
	}
      }

    }
  }

  if(seqBuffer.size > 0) {
    if( (polSta = PolLineToBuffer(&fileBuffer,"=SEQUENCE\n")) != POLOK ) {
      PolDisplayError(polSta,"PolLineToBuffer");
      Errorf("");
    }
    if( (polSta = _PolSeqBufferToFileBuffer(&fileBuffer,&seqBuffer)) != POLOK ) {
      PolDisplayError(polSta,"_PolSeqBufferToFileBuffer");
      Errorf("");
    } 
  }

  /* We can know put all this in a file. OUF... */
  if( (polSta = PolBufferToFile(fileName,&fileBuffer) ) != POLOK ) {
    PolDisplayError(polSta,"PolBufferToFile");
    Errorf("");
  }
}


/****************************************************************************/
/* READ VALUE CORRESPONDING TO fieldI IN FILE speciesPath/num AND COPY THE  */
/* RESULT IN varI                                                           */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/
void C_PolRead(char **argv)
{
  char *fileName;
  char fieldList[POLSIZELIST][POLSTRSIZE],varList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  char fieldExistVar[POLSTRSIZE];
  char *tempVar,*codage;
  int i,polSta,fieldExistFlag = 1;
  int size;
  char **listTemp;
  SIGNAL signal;

  fieldExistVar[0] = '\0';

  argv = ParseArgv(argv,tSTR,&fileName,-1);
  
  PolInitList(fieldList);
  PolInitList(varList);
  PolInitList(valueList);

    /* WE READ THE ARGUMENTS */
  if((polSta = PolOptArgs(argv,fieldList,varList)) != POLOK)
    {
      PolDisplayError(polSta,"PolOptArgs"); 
      Errorf("");
    }
  /*  NoMoreArgs(argv);*/
  /* ARGUMENTS HAVE BEEN READ */


  /* WE READ THE VALUES */
  if(( polSta = PolRead(fileName,fieldList,valueList)) != POLOK) 
     {
       PolDisplayError(polSta,"PolRead");
       Errorf("");
     }


  /* WE VERIFY THAT varList[i] IS CORRECT WHITH RESPECT TO fieldList[i] */
  /* IF SO WE PUT THE VALUES IN THE VARIABLES */
  for(i=0;(fieldList[i][0] != '\0') && (i < POLSIZELIST);i++)
    {
      if(!strcmp(fieldList[i],"F")) 
	{ /* WE TREAT THE SPECIAL OPTION F */
	  ParseList(varList[i],&listTemp);
	  ParseArgv(listTemp,tVNAME,&tempVar,0);
	  /*	  DeleteList(listTemp); */
	  
	  PolStrCpy(fieldExistVar,tempVar);
	  /*	  DeleteStr(tempVar);*/
	  /* AND GOTO NEXT FIELD */
	  continue;
	}
      else if( valueList[i][0] == '\0' ) 
	{ /* THE CORRESPONDING field DOESN'T EXIST IN speciesPath/num */
	  Printf("I didn't find the field: %s in file %s\n",fieldList[i],fileName); 
	  fieldExistFlag = 0;
	  /* SO WE GOTO NEXT FIELD */
	  continue;
	}


      if(!strcmp(fieldList[i],"SEQUENCE")) 
	{
	  ParseList(varList[i],&listTemp);
	  ParseArgv(listTemp,tSIGNAL,&signal,tSTR,&codage,0);
	  /*	  DeleteList(listTemp);*/
	  
	  if(strcmp(codage,"ag") && strcmp(codage,"ac") && strcmp(codage,"at")
	     && strcmp(codage,"a") && strcmp(codage,"c")
	     && strcmp(codage,"g") && strcmp(codage,"t")
	     && strcmp(codage,"Courb") && strcmp(codage,"courb"))
	    Errorf("Bad codage %s",codage);

	  /* EVERY THING IS OK */
	  /* WE READ THE SEQUENCE AND PUT IT IN signal */

	  /* ON MET LA SEQUENCE DANS un polBuffer */
	  if ((polSta = PolSequenceToBuffer(fileName,&seqBuffer)) != POLOK)
	    {
	      PolDisplayError(polSta,"PolSequenceToBuffer()");
	      Errorf("");
	    }
	  size = seqBuffer.size;
	  if(!strcmp(codage,"Courb") || !strcmp(codage,"courb"))
	    SizeSignal(signal,size-2,YSIG);
	  else
	    SizeSignal(signal,size,YSIG);
	  signal->x0 = 1;
	  signal->dx = 1;
	  
	  if((polSta = PolBufferToSignal(&seqBuffer,signal->Y,codage)) != POLOK) 
	    {
	      PolDisplayError(polSta,"PolBufferToSignal()");
	      Errorf("");
	    }
	}

      else 
	{ /* GENERIC CASE: TO A FIELD THERE CORRESPOND ONE VARIABLE */
	
	
	  ParseList(varList[i],&listTemp);
	  ParseArgv(listTemp,tSTR_,NULL,&tempVar,0);
	  /*	  DeleteList(listTemp);*/

	  if(tempVar != NULL)
	    SetStrVariable(tempVar,valueList[i]);
	
	  /*	  DeleteStr(tempVar);*/
	  AppendResultf("=%s %s\n",fieldList[i],valueList[i]);
	}
    }
  
  if(fieldExistVar[0] != '\0')
    SetStrVariable(fieldExistVar,(fieldExistFlag == 0) ? "0" : "1");
}

/****************************************************************************/
/* WRITE VALUE CORRESPONDING TO fieldI IN FILE speciesPath/num              */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/ 

void C_PolWrite(char **argv)
{
  char *fileName;
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  int polSta;


  argv = ParseArgv(argv,tSTR,&fileName,-1);
  
  PolInitList(fieldList);
  PolInitList(valueList);

  /* WE READ THE ARGUMENTS */
  if((polSta = PolOptArgs(argv,fieldList,valueList)) != POLOK)
    {
      PolDisplayError(polSta,"PolOptArgs"); 
      Errorf("");
    }
  /*  NoMoreArgs(argv);*/
  /* ARGUMENTS HAVE BEEN READ */


  /* WE WRITE THE VALUES */
  if((polSta = PolWrite(fileName,fieldList,valueList)) != POLOK) 
    {
      PolDisplayError(polSta,"PolWrite"); 
      Errorf("");
    }
  
}

void C_PolAddField(char **argv)
{
  char *fileName;
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  int polSta;

  argv = ParseArgv(argv,tSTR,&fileName,-1);
  
  PolInitList(fieldList);
  PolInitList(valueList);

  /* WE READ THE ARGUMENTS */
  if((polSta = PolOptArgs(argv,fieldList,valueList)) != POLOK)
    {
      PolDisplayError(polSta,"PolOptArgs"); 
      Errorf("");
    }
  /* ARGUMENTS HAVE BEEN READ */


  /* WE WRITE THE FIELDS */
  if((polSta = PolAddField(fileName,fieldList)) != POLOK)
    {
      PolDisplayError(polSta,"PolAddField"); 
      Errorf("");
    }
}

void C_PolRemoveField(char **argv)
{
  char *fileName;
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  int polSta;


  argv = ParseArgv(argv,tSTR,&fileName,-1);
  
  PolInitList(fieldList);
  PolInitList(valueList);

  /* WE READ THE ARGUMENTS */
  if((polSta = PolOptArgs(argv,fieldList,valueList)) != POLOK)
    {
      PolDisplayError(polSta,"PolOptArgs"); 
      Errorf("");
    }
  /* ARGUMENTS HAVE BEEN READ */


  /* WE REMOVE THE FIELDS */
  if((polSta = PolRemoveField(fileName,fieldList)) != POLOK)
    {
      PolDisplayError(polSta,"PolRemoveField"); 
      Errorf("");
    }
}

void C_PolReadSeq(char **argv)
{
  char *file;
  char *var;
  int polSta;

  argv = ParseArgv(argv,tSTR,&file,tVNAME,&var,0);

  /* THE buffer IS INITIALIZE IN PolSequenceToBuffer */
  if( (polSta = PolSequenceToBuffer(file,&seqBuffer)) != POLOK )
    {
      PolDisplayError(polSta,"PolSequenceToBuffer");
      Errorf("");
    }

  SetStrVariable(var,seqBuffer.buffer);
}

void C_PolReadRawSeq(char **argv)
{
  char *filename;
  int polSta;
  STREAM stream;
  LONGSTRING seqLS;

  argv = ParseArgv(argv,tSTREAM_,NULL,&stream,-1);
  if (stream == NULL) {
    argv = ParseArgv(argv,tSTR,&filename,-1);
    stream = OpenFileStream(filename,"r");
  }
  NoMoreArgs(argv);
 
  /* Some basic tests on the stream  */
  if (stream->mode != StreamRead) 
    Errorf("C_PolReadRawSeq() : The stream should be an input stream and not an output stream"); 
  if (stream->stream == NULL) 
    Errorf("C_PolReadRawSeq() : You cannot read a raw sequence from the standard input"); 

  seqLS = LSNew();
  if( (polSta = PolSeqToLongString(stream->stream,seqLS)) != POLOK )
    {
      LSDelete(seqLS);
      PolDisplayError(polSta,"PolSeqToLongString()");
      Errorf("");
    }

  SetResultStr(LSGetString(seqLS));

  LSDelete(seqLS);
}

void C_PolWriteSeq(char **argv)
{
  char *file;
  char *seq;;
  int polSta;
  FILE *fpFile;

  argv = ParseArgv(argv,tSTR,&file,tSTR,&seq,0);

  /* LET'S INITIALIZE THE buffer */
  fileBuffer.size = 0;
  fileBuffer.buffer[0] = '\0';
  seqBuffer.size = 0;
  seqBuffer.buffer[0] = '\0';

  /* Does the file exists ?  */
  if ( (fpFile = FOpen(file,"r")) != NULL ) {
    FClose(fpFile);
    /* Yes, then we replace its sequence by the sequence */
    if( (polSta = PolLineToBuffer(&fileBuffer,seq)) != POLOK ) {
      PolDisplayError(polSta,"PolLineToBuffer");
      Errorf("");
    }
    
    if( (polSta = PolSeqBufferToFile(file,&fileBuffer)) != POLOK ) {
      PolDisplayError(polSta,"PolSeqBufferToFile");
      Errorf("");
    }
  } else {
    /* No, then we create a new one */
    if( (polSta = PolLineToBuffer(&seqBuffer,seq)) != POLOK ) {
	  PolDisplayError(polSta,"PolLineToBuffer");
	  Errorf("");
    }
    if( (polSta = PolLineToBuffer(&fileBuffer,"=SEQUENCE\n")) != POLOK ) {
      PolDisplayError(polSta,"PolLineToBuffer");
      Errorf("");
    }
    if( (polSta = _PolSeqBufferToFileBuffer(&fileBuffer,&seqBuffer)) != POLOK ) {
      PolDisplayError(polSta,"_PolSeqBufferToFileBuffer");
      Errorf("");
    } 
    /* We can know put all this in a file. OUF... */
    if( (polSta = PolBufferToFile(file,&fileBuffer) ) != POLOK ) {
      PolDisplayError(polSta,"PolBufferToFile");
      Errorf("");
    }
  }

}

void C_PolSeqExtract(char **argv)
{
  char *seqIn,*seqOut;
  int deb,fin,size,i;

  argv = ParseArgv(argv,tSTR,&seqIn,tINT,&deb,tINT,&fin,0);

  if(deb<=0 || deb>fin)
    Errorf("We should have 0<deb<=fin");
  
  if(strlen(seqIn) < fin)
    Errorf("fin is to big");

  size = fin-deb+1;
  seqOut = CharAlloc(size+1);
  TempPtr(seqOut);
  for(i=0;i<size;i++)
    seqOut[i] = seqIn[deb-1+i];
  seqOut[size] = '\0';

  SetResultStr(seqOut);
}

void C_PolSimilitude(char **argv)
{
  char *seqA,*seqB;
  int n;

  argv = ParseArgv(argv,tSTR,&seqA,tSTR,&seqB,tINT,&n,0);

  if( (n<1) || (n>=POLSTRSIZE) )
    Errorf("n must be in the range [1..%d]",POLSTRSIZE-1);

  SetResultFloat(PolSimilitude(seqA,seqB,n));
}

void C_PolOppStrandSequence(char **argv)
{
  char *fileName;
  int polSta;

  argv = ParseArgv(argv,tSTR,&fileName,0);
  
  if( (polSta = PolOppStrandSequence(fileName)) != POLOK )
    {
      PolDisplayError(polSta,"PolOppStrandSequence()");
      Errorf("");
    }
  
}

void C_PolHelix3D(char **argv)
{
  char *file;
  int size,polSta;
  SIGNAL signalX,signalY,signalZ;

  argv = ParseArgv(argv,tSTR,&file,tSIGNAL,&signalX,tSIGNAL,
		   &signalY,tSIGNAL,&signalZ,0);

  /* THE buffer IS INITIALIZE IN PolSequenceToBuffer */
  if( (polSta = PolSequenceToBuffer(file,&seqBuffer)) != POLOK ) {
      PolDisplayError(polSta,"PolSequenceToBuffer");
      Errorf("");
    }
  size = seqBuffer.size;

  SizeSignal(signalX,size,YSIG);
  signalX->x0 = 0;
  signalX->dx = 1;
  signalX->firstp = 1;
  signalX->lastp = size - 2;

  SizeSignal(signalY,size,YSIG);
  signalY->x0 = 0;
  signalY->dx = 1;
  signalY->firstp = 1;
  signalY->lastp = size - 2;

  SizeSignal(signalZ,size,YSIG);
  signalZ->x0 = 0;
  signalZ->dx = 1;
  signalZ->firstp = 1;
  signalZ->lastp = size - 2;

  if( compute_all(seqBuffer.buffer,signalX->Y,signalY->Y,signalZ->Y,size) != 0)
    Errorf("Pb in C_PolHelix3D");
}

void C_PolAngle(char **argv)
{
  int size,sizeSig;
  int polSta,size_window;
  SIGNAL signal;
  char *seq;

  argv = ParseArgv(argv,tSTR,&seq,tSIGNAL,&signal,tINT,&size_window,0);

  /* Copy the sequence to the seqBuffer */
  seqBuffer.size = 0;
  seqBuffer.buffer[0] = '\0';
  if( (polSta = PolLineToBuffer(&seqBuffer,seq)) != POLOK ) {
    PolDisplayError(polSta,"PolLineToBuffer");
    Errorf("");
  }

  sizeSig = seqBuffer.size;
  SizeSignal(signal,sizeSig,YSIG);
  signal->x0 = 0;
  signal->dx = 1;
  
  if( compute_all_angle(seqBuffer.buffer,signal->Y,signal->size,size_window) != 0)
    Errorf("Pb in C_PolAngle");
}

void C_PolCourb(char **argv)
{
  int size_seq;
  int polSta;
  SIGNAL signal;
  char *seq;

  argv = ParseArgv(argv,tSTR,&seq,tSIGNAL,&signal,0);

  size_seq = strlen(seq);
  SizeSignal(signal,size_seq-2,YSIG);
  signal->x0 = 0;
  signal->dx = 1;
  
  if((polSta = PolStringToSignal(seq,size_seq,signal->Y,"courb")) != POLOK) {
    if(polSta == POLERROR) {
      Errorf("PolStringToSignal(): Their must have been a problem while "
	     "reading one of the roll or tilt files. Try to respecify them "
	     "using 'pol helix'\n.");
    }
    PolDisplayError(polSta,"PolBufferToSignal()");
    Errorf("");
  }
}

void C_PolHelixFile(char **argv)
{
 char *action;
 char *filename;
 
 argv = ParseArgv(argv,tWORD,&action,tSTR_,NULL,&filename,0);
 
 /* 'roll' action */
  if (!strcmp(action,"roll")) 
    {
      if(change_roll_file(filename) != 0)
	Errorf("Pb with change_roll_file");
    }
  /* 'tilt' action */
  else if (!strcmp(action,"tilt"))
   {
     if(change_tilt_file(filename) != 0)
       Errorf("Pb with change_tilt_file");
   }
  else
      Errorf("Unknow action '%s'",action);
}

void C_PolRanSeq(char **argv)
{
  FLOAT a,t,g;
  int size,polSta;
  char *vname;

  argv = ParseArgv(argv,tINT,&size,
		   tFLOAT,&a,tFLOAT,&t,tFLOAT,&g,0);

  /* THE seqBuffer IS INITIALIZE IN create_seq_one_h */
  if (ran_seq(&seqBuffer,size,a,t,g) != 0)
    Errorf("Pb in ran_seq");

  SetResultStr(seqBuffer.buffer);
}

/* Function in seq_create.c */
extern int courb_one_h(PolluxBuffer *ranSeqBuffer,FLOAT H,int size,
		       FLOAT courb_ref_val);
void C_PolCourbOneH(char **argv)
{
  FLOAT H,courb_ref_val;
  int size,polSta;
  
  argv = ParseArgv(argv,tFLOAT,&H,tINT,&size,tFLOAT,&courb_ref_val,0);

  /* THE seqBuffer IS INITIALIZE IN create_seq_one_h */
  if (courb_one_h(&seqBuffer,H,size,courb_ref_val) != 0)
    Errorf("Pb in create_seq_one_h");

  SetResultStr(seqBuffer.buffer);
}

/* Function in seq_create.c */
extern int mononuc_one_h(PolluxBuffer *ranSeqBuffer,FLOAT H,int size,
			 char *set1,char *set2,FLOAT c1);
void C_PolMonoNucOneH(char **argv)
{
  FLOAT H,p1;
  int size,polSta;
  char *set1,*set2;
  
  argv = ParseArgv(argv,tFLOAT,&H,tINT,&size,tFLOAT,&p1,
		   tSTR_,"ag",&set1,tSTR_,"ct",&set2,0);

  /* THE seqBuffer IS INITIALIZE IN create_seq_one_h */
  if (mononuc_one_h(&seqBuffer,H,size,set1,set2,p1) != 0)
    Errorf("Pb in mono_nuc_one_h() (One of the parameter must be out of range).");

  SetResultStr(seqBuffer.buffer);
}

/* Function in seq_create.c */
extern int mononuc_two_h(PolluxBuffer *seqBuffer,FLOAT h,FLOAT H,
			 int noct,int size,
			 char *set1,char *set2,FLOAT c1,char *filename);
void C_PolMonoNucTwoH(char **argv)
{
  FLOAT h,H,p1,dp1;
  int noct,size,polSta;
  char *set1,*set2;

  argv = ParseArgv(argv,tFLOAT,&h,tFLOAT,&H,
		   tINT,&noct,tINT,&size,
		   tFLOAT,&p1,
		   tSTR_,"ag",&set1,tSTR_,"ct",&set2,0);

  /* THE seqBuffer IS INITIALIZE IN mononuc_two_h_alt */
  if (mononuc_two_h(&seqBuffer,h,H,noct,size,set1,set2,p1,"d9h50.b") != 0)
    Errorf("Pb in mononuc_two_h_alt() (One of the parameter must be out of range).");

  SetResultStr(seqBuffer.buffer);
}

/* Function in seq_create.c */
extern int mononuc_two_h_old(PolluxBuffer *seqBuffer,FLOAT h,FLOAT H,int size_w,int num_repeat,
			     char *set1,char *set2,FLOAT c1, FLOAT dc1);
void C_PolMonoNucTwoHOld(char **argv)
{
  FLOAT h,H,p1,dp1;
  int size_w,num_repeat,polSta;
  char *set1,*set2;
  
  argv = ParseArgv(argv,tFLOAT,&h,tFLOAT,&H,
		   tINT,&size_w,tINT,&num_repeat,
		   tFLOAT,&p1,tFLOAT,&dp1,
		   tSTR_,"ag",&set1,tSTR_,"ct",&set2,0);

  /* THE seqBuffer IS INITIALIZE IN mononuc_two_h */
  if (mononuc_two_h_old(&seqBuffer,h,H,size_w,num_repeat,set1,set2,p1,dp1) != 0)
    Errorf("Pb in mononuc_two_h() (One of the parameter must be out of range).");

  SetResultStr(seqBuffer.buffer);
}

void C_PolPolyNuc(char **argv)
{
  char **codList;
  SIGNAL signal;
  int polyLength,polSta,size_seq;
  char *seq;

  argv = ParseArgv(argv,tSTR,&seq,tSIGNAL,&signal,
		   tINT,&polyLength,tLIST,&codList,0);
  
  /* SOME CHECKING */
  if(polyLength <= 0)
    Errorf("polyLength should be > 0 !!");

  size_seq = strlen(seq);
  /* LET'S BUILD THE SIGNAL */
  SizeSignal(signal,size_seq-polyLength+1,YSIG);
  signal->x0 = (polyLength-1.)/2.;
  signal->dx = 1.;

  if( (polSta = PolCodPolyNucStringToSignal(seq,size_seq,signal->Y,
					    codList,polyLength))
      != POLOK ) {
    PolDisplayError(polSta,"PolCodPolyNucStringToSignal");
    Errorf("");
  }  
}

#define DISTTABSIZEMAX 20
void C_PolDistPolyNuc(char **argv)
{
  char opt;
  int flagTypeFill;
    char **distList,**codList1,**codList2;
  SIGNAL signal;
  int polyLength,polSta;
  int distNum,distPrev;
  int distTab[DISTTABSIZEMAX];
  char *seq;

  argv = ParseArgv(argv,tSTR,&seq,tSIGNAL,&signal,
		   tINT,&polyLength,tLIST,&distList,
		   tLIST,&codList1,tLIST,&codList2,-1);
  
  flagTypeFill = POLFILLMOTIF;
  while(opt = ParseOption(&argv))
    {
      switch(opt) 
        {
	case 'f': /* flagTypeFill */
	  flagTypeFill = POLFILLFULL;
	  break;      
	case 'p': /* flagTypeFill */
	  flagTypeFill = POLFILLPOS;
	  break;      
        default:
          ErrorOption(opt);
        }
    }
  NoMoreArgs(argv);

  /* SOME CHECKING */
  if(polyLength <= 0)
    Errorf("polyLength should be > 0 !!");
  
  /* Reading distances */
  distNum = 0;
  distPrev = 0;
  while(*distList)
    {
      ParseInt(*distList,distTab+distNum);
      if(distTab[distNum] <= distPrev)
	Errorf("distance should be >0 and given in increasing order !!");
      distPrev = distTab[distNum];

      distNum++;
      if( distNum >= DISTTABSIZEMAX )
	Errorf("Distance list is too long !");
      
      distList++;
    }
  

  /* Copy the sequence to the seqBuffer */
  seqBuffer.size = 0;
  seqBuffer.buffer[0] = '\0';
  if( (polSta = PolLineToBuffer(&seqBuffer,seq)) != POLOK ) {
    PolDisplayError(polSta,"PolLineToBuffer");
    Errorf("");
  }
  
  /* LET'S BUILD THE SIGNAL */
  SizeSignal(signal,seqBuffer.size,YSIG);
  signal->x0 = 1.;
  signal->dx = 1.;

  if( (polSta = PolDistantPolyNucBufferToSignal(&seqBuffer,signal->Y,
						codList1,codList2,
						polyLength,distTab,distNum,
						flagTypeFill)
       )
      != POLOK )
    {
      PolDisplayError(polSta,"PolDistantPolyNucBufferToSignal()");
      Errorf("");
    }  
}

void C_Farima(char **argv) {
    SIGNAL signal;
    int size;
    FLOAT H,sigma;
    
    argv = ParseArgv(argv,tINT,&size, tFLOAT, &H, tFLOAT, &sigma,0);
    
    /* LET'S BUILD THE SIGNAL */
    signal = TNewSignal();
    SizeSignal(signal,size,YSIG);
    signal->x0 = 1.;
    signal->dx = 1.;
    
    Farima(signal,size,H,sigma);
    
    SetResultValue(signal);
}


void C_Pol(char **argv)
{
  char *action;

  argv = ParseArgv(argv,tWORD,&action,-1);
  
  /* 'read' action */
  if (!strcmp(action,"read")) 
    C_PolRead(argv);
  
  /* 'readall' action */
  else if (!strcmp(action,"readall"))
    {
      C_PolReadAll(argv);
    }

  /* 'speciesreadall' action */
  else if (!strcmp(action,"speciesreadall"))
    {
      C_PolSpeciesReadAll(argv);
    }

  /* 'writeall' action */
  else if (!strcmp(action,"writeall"))
    {
      C_PolWriteAll(argv);
    }

  /* 'write' action */
  else if (!strcmp(action,"write"))
    {
      C_PolWrite(argv);
    }

  /* 'addfield' action */
  else if (!strcmp(action,"addfield"))
    {
      C_PolAddField(argv);
    }

  /* 'removefield' action */
  else if (!strcmp(action,"removefield"))
    {
      C_PolRemoveField(argv);
    }

  /* 'extract' action */
  else if (!strcmp(action,"extract"))
    {
      C_PolExtract(argv);
    }

  /* 'file2pathname' action */
  else if (!strcmp(action,"file2pathname"))
    {
      C_PolFileToSpeciesNum(argv);
    }
  
  /* 'readseq' action */
  else if (!strcmp(action,"readseq")) 
    {
      C_PolReadSeq(argv);
    }

  /* 'readrawseq' action */
  else if (!strcmp(action,"readrawseq")) 
    {
      C_PolReadRawSeq(argv);
    }
  
  /* 'writeseq' action */
  else if (!strcmp(action,"writeseq")) 
    {
      C_PolWriteSeq(argv);
    }
  
  /* 'sim' action */
  else if (!strcmp(action,"sim")) 
    {
      C_PolSimilitude(argv);
    }
  
  /* 'oppstrand' action */
  else if (!strcmp(action,"oppstrand")) 
    {
      C_PolOppStrandSequence(argv);
    }

  /* 'seqextract' action */
  else if (!strcmp(action,"seqextract")) 
    {
      C_PolSeqExtract(argv);
    }
  
  /* 'subsample' action */
  else if (!strcmp(action,"subsample")) 
    {
      C_PolSubSampleSequence(argv);
    }
  
  /* 'angle' action */
  else if (!strcmp(action,"angle")) 
    {
      C_PolAngle(argv);
    }

  /* 'courb' action */
  else if (!strcmp(action,"courb")) 
    {
      C_PolCourb(argv);
    }

  /* 'helix' action */
  else if (!strcmp(action,"helix")) 
    {
      C_PolHelix3D(argv);
    }

  /* 'helix_file' action */
  else if (!strcmp(action,"helix_file")) 
    {
      C_PolHelixFile(argv);
    }

  /* 'lrc_courb' action */
  else if (!strcmp(action,"lrc_courb")) 
    {
      C_PolCourbOneH(argv);
    }

  /* 'lrc_mononuc' action */
  else if (!strcmp(action,"lrc_mononuc")) 
    {
      C_PolMonoNucOneH(argv);
    }

  /* 'lrc2_mononuc' action */
  else if (!strcmp(action,"lrc2_mononuc")) 
    {
      C_PolMonoNucTwoH(argv);
    }

  /* 'lrc2_mononucold' action */
  else if (!strcmp(action,"lrc2_mononucold")) 
    {
      C_PolMonoNucTwoHOld(argv);
    }

  /* 'codpolynuc' action */
  else if (!strcmp(action,"codpolynuc"))
    {
      C_PolPolyNuc(argv);
    }

  /* 'distpolynuc' action */
  else if (!strcmp(action,"distpolynuc"))
    {
      C_PolDistPolyNuc(argv);
    }

  /* 'ranseq' action */
  else if (!strcmp(action,"ranseq"))
    {
      C_PolRanSeq(argv);
    }

  else
    Errorf("Unknow action '%s'",action);
}


static CProc polluxCommands[] = {
  "fbm",C_Farima,
  "{{{<size> <H> <sigma>} {Computes a fractional brownian motion of Hurst exponent <H> using the integration of a fractional ARIMA.}}}",
  
  "pol",C_Pol,

  "{{speciesreadall <speciename> <array> {-i <imax> | -s <signal> | -S}} "
  "{Calls "
  "readall for all (-i) or part (-s) of files of a specie (but in C)\n"
  "  -S: load the sequence as well. (The default is not to load "
  "the sequence.)}} "

  "{{readall <filename> <array> [-S]} "
  "{Clears the array and set variables acccording to the pollux "
  "file <filename>.\n"
  "  -S: load the sequence as well. (The default is not to load "
  "the sequence.)}} "

  "{{writeall <filename> <array>} "
  "{Write the content of <array> to a file in Pollux format. "
  "(All the elements in the array have to be strings.)}} "
  
  "{{read <filename> list of {SEQUENCE <signal> <codage> } | "
  "{<field> <varname>} | [{F <varname>}] } {"
  " - reads a sequence, codes it according to <cogage> and put the "
  "result in <signal>;\n"
  " - reads fields and put the result in variable <varname>;\n"
  " - the special field F checks if all fiels exist and set <varname> "
  "accordingly (0 or 1).}} "

  "{{extract <archivename> <speciespath>} "
  "{extracts all the file from a pollux archive}} "

  "{{addfield <filename> list of <field>} {add new fields}} "
  "{{removefield <filename> list of <field>} {remove fields}} "

  "{{write <filename> list <field> <value>} "
  "{write a new value to an existing field.}} "

  "{{file2pathname <filename> <varpath> <varname>} "
  "{determines the directory and the name of a unix-type filepath}} "

  "{{readseq <filename> <varname>} "
  "{Reads a DNA sequence from a pollux file and put it in the string "
  "variable <varname>.}} "

  "{{readrawseq <filename>|<stream>} "
  "{Returns the DNA sequence in raw format read from a  file or a stream.}} "

  "{{writeseq <filename> <varname>} "
  "{If the pollux file <filename> exists, it replace its sequence by "
  "the sequence in string variable <varname> otherwise it creates a "
  "new pollux file.}} "

  "{{seqextract <seqIn> <deb> <fin>} "
  "{Obsolete: Return the subsequence from pos <deb> "
  "to pos <fin> of <seqIn>.}} "

  "{{sim <varnameA> <varnameB> <n>} {Computes the degree of likeness of "
  "two sequences, it counts the number of words of lengh n of sequence A "
  "(in varA) that also exist in sequence B (in bufferB) }} "

  "{{oppstrand <pollux file>} {It compute the sequence corresponding "
  "to the opposite strand (reverse complement).}} "

  "{{subsample <fileNameIn> <fileNameOut> <phase>} {It Subsample the sequence "
  "in <fileNameIn>, in order to obtain only nucleotide in <phase> position "
  "within their codon}} "

  "{{angle <vname> <signal> <sizeWindow>} {return the cosinus of the angle made "
  "by the DNA helix between positions at a distance <sizeWindow> (in bp). It depends on "
  "the roll and tilt angle specified (see pol helix_file).}} "

  "{{courb <vname> <signal>} {return the local curvature "
  "of the DNA helix (in arbitrary units). It depends on "
  "the roll and tilt angle specified (see pol helix_file).\n"
  "Example: pol helix_file roll "
  "$Home/LastWave/scripts/pollux/roll_[pnuc|dnase].dat\n"
  "         pol courb dna 0a\n"
  "         Computes the [PNuc|DNase] coding from JMB 316, 903-918 (2002).}} "

  "{{helix <polfile> <signalX> <signalY> <signalZ>} {Returns the X, Y, and Z "
  "coordinates of the DNA helical axis at each base pair. It depends on "
  "the roll and tilt angle specified (see pol helix_file).}} "

  "{{helix_file {roll | tilt} <filename>} {Enables to specifies the roll or tilt "
  " angle file that specifies the geometrical modeling of the DNA helical axis. "
  "Note that we use a constant twist of 34.3 degree and no translation such as "
  "the rise.}} "

  "{{lrc_courb <H> <size> <courb_mean_val>} {Returns a DNA sequences "
  "whose bending profile is long-range correlated with a Hurst exponent H. "
  "It depends on the roll and tilt angle specified (see pol helix_file).}} "

  "{{lrc_mononuc <H> <size> <p1> [<set1>=ag] [<set2>=ct]} "
  "{Returns a DNA sequence long-range correlated with a Hurst exponent H "
  "for the mononucleotide distinction <set1>/<set2> and with composition "
  "p1 for nucleotide from <set1>.}} "

  "{{lrc2_mononuc <h> <H> <noct> <size> <p1> [<set1>=\"ag\"] "
  "[<set2>=\"ct\"]} "
  "{Returns a long-range correlated DNA sequence with two scaling regimes. "
  "The small scale regime goes up to ~2^(noct-1) and is characterize by a "
  "Hurst exponent <h>. "
  "The large scale regime start at ~2^noct and is characterize by a Hurst "
  "exponent <H>. "
  "The total size of the generated sequence is <size>. "
  "The correlations are forced to the mononucleotide distinction "
  "<set1>/<set2>. The base composition can be adjusted by <p1> which "
  "is the compostion in nucleotide from <set1>.\n"
  "Example: pol lrc2_mononuc 0.6 0.8 0.4 \"gc\" \"at\"\n"
  "         Synthesise a DNA sequence with 40% G+C and impose two scaling\n"
  "         regime on the weak/strong distribution of nucleotide.}} "

  /*  "{{lrc2_mononucold <vname> <h> <H> <size_w> <num_repeat> <p1> <dp1> [<set1>=ag] [<set2>=ct]} "
      "{Computes a long-range correlated DNA sequence with two scaling regimes. "
      "The small scale regime goes up to <size_w> and is characterize by a Hurst exponent <h>. "
      "The large scale regime start at <size_w> and is characterize by a Hurst exponent <H>. "
      "The total size of the generated sequence is <size_w*num_repeat>. "
      "The correlations are forced to the mononucleotide distinction <set1>/<set2>. "
      "The base composition can be adjusted by <p1> which is the compostion in nucleotide "
      "from <set1>. Finally, <dp1> is the composition fluctuation that control the relative "
      "amplitude of the two scaling regime (0<p1-dp1 and p1+dp1<1).}} "
  */

  "{{codpolynuc <seq> <signalOut> <polyLength> <list of polynucleotides>} "
  "{Computes the caracteristic signal corresponding to a polynucleotide list "
  "of unique length <polyLength> from a polux file.\n"
  "Example: pol codpolynuc dna 0a 2 \"aa tt\"\n"
  "         Computes the AA(=TT) coding from JMB 316, 903-918 (2002). }} "

  "{{distpolynuc <seq> <signalOut> <polyLength> <polydist List> "
  "<first list of polynucleotides> <second list of polynucleotides> "
  "[-f] [-p]} "
  "{Compute the caracteristic signal corresponding to motifs made of 2 "
  "submotifs corresponding to polynucleotide lists of a unique length "
  "<polyLength> and <polydist>bp distant apart (measured from the beginning"
  "of each submotif). "
  "The default is to put 1 on each of the nucleotide matching the motif; "
  "-f: fill the motif, -p: just mark its middle position.\n"
  "Example: pol distpolynuc dna 0a 2 \"10 11\" \"aa tt\" \"aa tt\"\n"
  "         Marks all motifs composed of dinucleotide aa or tt, 10 or 11 bp\n"
  "         away from another aa or tt.}} "

  "{{ranseq <size> <ca> <ct> <cg>} "
  "{Returns a random sequences of a given composition}} ",
 
  NULL,NULL,NULL
};
CProcTable polluxTable = {polluxCommands,"pollux","Commands to deal with DNA sequences"};

