/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

/****************************************************************************/
/*                                                                          */
/*                         READ POLLUX FILES                                */
/*                                                                          */
/****************************************************************************/



#include "pollux.h"
#include "helix_3d.h"
#include <math.h>
#include <stdlib.h>
/****************************************************************************/
/*   EXTRACT ALL fields IN FILE filName, PUT ITS VALUE IN                   */
/*   (**char) fieldList, READ THE CORRESPONDING                             */
/*   values AND PUT THEM IN valueList (**char)                              */
/*   IF seqBuffer != NULL IT COPIES THE SEQUENCE TO seqBuffer               */
/****************************************************************************/
/* fieldList AND valueList ARE SUPOSED TO BE char [POLSIZELIST][POLSTRSIZE] */
/* seqBuffer IS SUPPOSED TO HAVE BEEN INITIALIZED                           */
/****************************************************************************/
int PolReadAll(FILE *fpFile,char fieldList[][POLSTRSIZE],
	       char valueList[][POLSTRSIZE],PolluxBuffer *seqBuffer)
{
  char line[POLSTRSIZE],field[POLSTRSIZE],value[POLSTRSIZE];
  int i,sta,flagSeq = POLNO;


  /* THE FILE WILL BE TREATED LINE BY LINE */
  i = 0;
  while(fgets(line,POLSTRSIZE,fpFile)!=NULL)
    {
      sta = sscanf(line," =%s %[^\n\r]",field,value);

      if (sta < 1) {
	/* THE LINE IS NOT A DEFINITION: GOTO NEXT LINE */
	continue;
      }
      else 
	{ 
	  /* AT LEAST field IS DEFINED */
	  if (!strcmp(field,"SEQUENCE"))  /* WE REACHED THE END OF THE DEFINITION ZONE: IT IS FINISHED */
	    {
	      flagSeq = POLYES;
	      break;
	    }
	  
	  if (i >= POLSIZELIST)
	    return POLOVERFLOW;

	  /* WE COPY field  TO fieldList */
	  /* THERE IS NO PB OF OVERFLOW BECAUSE  WE USED fgets */
	  strcpy(fieldList[i],field);
	  
	  /* IF sta!=2 THERE IS NO VALUE CORRESPONDING TO field NEVERTHELESS */
	  /* WE SET valueList[i] TO "" */
	  if(sta == 2)
	    {
	      /* LET'S REMOVE THE TRAILING BLANKS */
	      RemoveTrailBlank(value);
	      strcpy(valueList[i],value);
	    }
	  else valueList[i][0] = '\0';

	  i++;
	}
      
    }

  /* NOW fpfile POINTS TO THE BEGINING OF THE SEQUENCE ZONE */
  /* IF flagSeq == POLYES */
  if (flagSeq == POLYES && seqBuffer != NULL)
    if ((sta = PolSeqToBuffer(fpFile,seqBuffer)) != POLOK)
      return sta;

  return POLOK;
}

/****************************************************************************/
/*   EXTRACT ALL fields IN fieldList (**char) READ THE CORRESPONDING        */
/*   values IN FILE POINTED BY fpFile AND PUT THEM IN valueList (**char)    */
/****************************************************************************/
/*      */
/*                          */
/****************************************************************************/
int PolRead(const char *fileName, char fieldList[][POLSTRSIZE],
	    char valueList[][POLSTRSIZE])
{
  char line[POLSTRSIZE],field[POLSTRSIZE],value[POLSTRSIZE];
  int i,sta;
  FILE *fpFile;

  if ((fpFile = fopen(fileName,"r")) == NULL)
    {
      Printf("can't open file %s\n",fileName);
      return POLFILEIOERROR;
    }

  /* THE FILE WILL BE TREATED LINE BY LINE */
  while(fgets(line,POLSTRSIZE,fpFile)!=NULL)
    {
      sta = sscanf(line," =%s %[^\n\r]",field,value);

      if (sta < 1) {
	/* THE LINE IS NOT A DEFINITION: GOTO NEXT LINE */
	continue;
      }
      else 
	{ /* AT LEAST field IS DEFINED */
	  /* WE LOOK FOR field IN fieldList */
	  if(IsOfStringList(field,fieldList,&i) == POLYES)
	    { /* IF IT IS THERE THEN ITS CORRESPONDING VALUE IS COPIED IN valueList */
	      /* IF sta!=2 THERE IS NO VALUE CORRESPONDING TO field NEVERTHELESS */
	      /* WE SET valueList[i] TO " " TO SAY THAT THIS FIELD WAS FOUND */
	      if(sta == 2)
		{
		  /* LET'S REMOVE THE TRAILING BLANKS */
		  RemoveTrailBlank(value);
		  strcpy(valueList[i],value);
		}
	      else strcpy(valueList[i]," ");
	    }
	  
	  if (!strcmp(field,"SEQUENCE"))  /* WE REACHED THE END OF THE DEFINITION ZONE: IT IS FINISHED */
	    break;
	  
	  
	}
    }
  fclose(fpFile);
  return POLOK;
}

/****************************************************************************/
/*   EXTRACT THE SEQUENCE OF A POLLUX FILE fileName AND PUT THE             */
/*   CORRESPONDING WALK IN SIGNAL signal USING CODAGE codage                */
/****************************************************************************/
/*   THE SIZE OF THE ARRAY signal MUST BE AT LEAST polbuffer->size          */
/*                                                                          */
/****************************************************************************/
int PolBufferToSignal(PolluxBuffer *polbuffer,FLOAT *signal,char *codage) {
  return PolStringToSignal(polbuffer->buffer, polbuffer->size, signal, codage);
}

int PolStringToSignal(char *seq, int sizeb, FLOAT *signal,char *codage) {
  int i;
  FLOAT a,c,g,t,r,y,s,w,m,k,flag;

  if (!strcmp(codage,"ag")) {
    a = 1.0;
    g = 1.0;
    c = -1.0;
    t = -1.0;
    r = 1.0;
    y = -1.0;
    s = 0.0;
    w = 0.0;
    m = 0.0;
    k = 0.0;
  } else if(!strcmp(codage,"ac")) {
    a = 1.0;
    g = -1.0;
    c = 1.0;
    t = -1.0;
    r = 0.0;
    y = 0.0;
    s = 0.0;
    w = 0.0;
    m = 1.0;
    k = -1.0;
  } else if(!strcmp(codage,"at")) {
    a = 1.0;
    g = -1.0;
    c = -1.0;
    t = 1.0;
    r = 0.0;
    y = 0.0;
    s = -1.0;
    w = 1.0;
    m = 0.0;
    k = 0.0;
  } else if(!strcmp(codage,"a")) {
    a = 1.0;
    g = 0.0;
    c = 0.0;
    t = 0.0;
    r = 0.0;
    y = 0.0;
    s = 0.0;
    w = 0.0;
    m = 0.0;
    k = 0.0;
  } else if(!strcmp(codage,"c")) {
    a = 0.0;
    g = 0.0;
    c = 1.0;
    t = 0.0;
    r = 0.0;
    y = 0.0;
    s = 0.0;
    w = 0.0;
    m = 0.0;
    k = 0.0;
  } else if(!strcmp(codage,"g")) {
    a = 0.0;
    g = 1.0;
    c = 0.0;
    t = 0.0;
    r = 0.0;
    y = 0.0;
    s = 0.0;
    w = 0.0;
    m = 0.0;
    k = 0.0;
  } else if(!strcmp(codage,"t")) {
    a = 0.0;
    g = 0.0;
    c = 0.0;
    t = 1.0;
    r = 0.0;
    y = 0.0;
    s = 0.0;
    w = 0.0;
    m = 0.0;
    k = 0.0;
  } else if(!strcmp(codage,"Courb") || !strcmp(codage,"courb")) {
    FLOAT *X,*Y,*Z;
    FLOAT dx,dy,dz;
    
    X = (FLOAT *) malloc(3*sizeb*sizeof(FLOAT));
    if(X == NULL)
      return POLMEMERROR;
    Y = X+sizeb;
    Z = Y+sizeb;
    
    if(compute_all(seq,X,Y,Z,sizeb) != 0)	{
      free(X);
      return POLERROR;
    }

    for(i=0;i<sizeb-2;i++) {
      dx = X[i+1]-X[i];
      dy = Y[i+1]-Y[i];
      dz = Z[i+1]-Z[i];
      signal[i] = sqrt(dx*dx+dy*dy+dz*dz);
    }
    
    free(X);
    return POLOK;
  } else {
    return POLERROR;
  }

  /* WE CAN NOW CONVERT THE SEQUENCE TO A WALK */
  for(i=0;i<sizeb;i++) {
    switch(seq[i]) {
    case 'A':
    case 'a':
      flag = a;
      break;
    case 'T':
    case 't':
      flag = t;
      break;
    case 'G':
    case 'g':
      flag = g;
      break;
    case 'C':
    case 'c':
      flag = c;
      break;
    case 'R':
    case 'r':
      flag = r;
      break;
    case 'Y':
    case 'y':
      flag = y;
      break;
    case 'S':
    case 's':
      flag = s;
      break;
    case 'W':
    case 'w':
      flag = w;
      break;
    case 'M':
    case 'm':
      flag = m;
      break;
    case 'K':
    case 'k':
      flag = k;
      break;
    case 'B':
    case 'b':
    case 'H':
    case 'h':
    case 'D':
    case 'd':
    case 'V':
    case 'v':
    case 'N':
    case 'n':
      flag = 0.0;
      break;
    default:
      return POLFORMATERROR;
      break;
    }

    signal[i] = flag;
  }
  return POLOK;
}

int PolCodPolyNucBufferToSignal(PolluxBuffer *seqBuffer,FLOAT *signal,
				char **codList,int polyLength) {
  return PolCodPolyNucStringToSignal(seqBuffer->buffer, seqBuffer->size, signal, codList, polyLength);
}

int PolCodPolyNucStringToSignal(char *seq, int sizeb, FLOAT *signal,
				char **codList, int polyLength) {
  int i,sizeSig;
  char **list;
  
  /* SOME CHECKING */
  for(list=codList;*list!=NULL;list++) {
    if(strlen(*list) != polyLength)
      return POLFORMATERROR;
  }
  
  /* LET'S BUILD THE SIGNAL */
  sizeSig = sizeb-polyLength+1;
  for(i=0;i<sizeSig;i++) {
    signal[i] = 0.;
    for(list=codList;*list!=NULL;list++) {
      if(strncasecmp(*list,seq+i,polyLength) == 0) {
	signal[i] = 1.;
	break;
      }
    }
  }

  return POLOK;
}

int PolDistantPolyNucBufferToSignal(PolluxBuffer *seqBuffer,FLOAT *signal,
				    char **codList1,char **codList2,
				    int polyLength,int *distTab,int distNum,
				    int flagTypeFill)
{
  int i,p,d,dist,sizeI,sizeb=seqBuffer->size;
  char **list1,**list2;
  
  /* SOME CHECKING */
  for(list1 = codList1;*list1 != NULL; list1++)
    {
      if(strlen(*list1) != polyLength)
	return POLFORMATERROR;
    }
  for(list2 = codList2;*list2 != NULL; list2++)
    {
      if(strlen(*list2) != polyLength)
	return POLFORMATERROR;
    }
  for(d=0;d<distNum-1;d++)
    {
      /* We expect distTab to be sorted in increasing order */
      if(distTab[d] >= distTab[d+1])
	return POLFORMATERROR;
    }
  
  /* LET'S BUILD THE SIGNAL */
  /* We put a +1 for EACH nucleotides part of our pattern */
  for(i=0;i<sizeb;i++)
    signal[i] = 0.;
  
  for(d=0;d<distNum;d++)
    {
      dist = distTab[d];
      sizeI = sizeb-polyLength-dist+1;

      for(i=0;i<sizeI;i++)
	{

	  for(list1=codList1;*list1 != NULL;list1++)
	    {
	      if(strncasecmp(*list1,seqBuffer->buffer+i,polyLength) == 0)
		{

		  for(list2=codList2;*list2 != NULL;list2++)
		    {
		      if(strncasecmp(*list2,seqBuffer->buffer+i+dist,polyLength) == 0)
			{
			  if(flagTypeFill == POLFILLFULL)
			    {
			      for(p=0;p<polyLength+dist;p++)
				signal[i+p] = 1.;
			    }
			  else if(flagTypeFill == POLFILLMOTIF)
			    {
			      for(p=0;p<polyLength;p++)
				{
				  signal[i+p] = 1.;
				  signal[i+p+dist] = 1.;
				}
			    }
			  else if(flagTypeFill == POLFILLPOS)
			    {
			      signal[i+(dist+polyLength)/2] += 1.;
			    }
			  else 
			    return POLFORMATERROR;
			  break;
			}
		    }

		  break;	      
		}
	    }

	}

    }

  return POLOK;
}
