/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

#include "pollux.h"
#include <stdlib.h>

/****************************************************************************/
/* COPIES THE HEADER OF A POLLUX FILE TO A pol buffer.  */
/* */
/* */
/* */
/****************************************************************************/
/* THE buffer IS SUPPOSED TO HAVE BEEN INITIALIZED. */
/* */
/****************************************************************************/
static int _PolDefsToBuffer(FILE *fp,PolluxBuffer *fileBuffer)
{
  int sta,polSta;
  char line[POLSTRSIZE],field[POLSTRSIZE];

  /* LET'S COPY UP TO THE SEQUENCE */
  while(fgets(line,POLSTRSIZE,fp) != NULL)
    {
      /* WE COPY THE LINE TO THE BUFFER */
      if((polSta = PolLineToBuffer(fileBuffer,line)) != POLOK) 
	return polSta;

      sta = sscanf(line," =%s ",field);
      if (sta == 1)  /* field IS DEFINED */
	if (!strcmp(field,"SEQUENCE"))  
	  /* WE COPIED THE FILE UP TO THE END OF THE DEFINITION ZONE: return */
	  return POLOK;
    }

  return POLOK;
}
static void _PolJumpToSequence(FILE *fp)
{
  char line[POLSTRSIZE],field[POLSTRSIZE];
  int sta;

  while(fgets(line,POLSTRSIZE,fp) != NULL)
    {
      sta = sscanf(line," =%s ",field);
      
      /* THE LINE IS NOT A DEFINITION: GOTO NEXT LINE */
      if (sta < 1) 
	continue;
      else /* field IS DEFINED */ 
	if (!strcmp(field,"SEQUENCE")) 
	  /* WE REACHED THE BEGINING OF THE SEQUENCE ZONE */
	  return;
    }
  return;
}    
	    
/****************************************************************************/
/* COPIES THE SEQUENCE  TO A pol buffer.  */
/* */
/* */
/* */
/****************************************************************************/
/* THE buffer IS SUPPOSED TO HAVE BEEN INITIALIZED. */
/* fp IS SUPPOSED TO POINT TO THE BEGINING OF THE SEQUENCE ZONE */
/****************************************************************************/
int PolSeqToBuffer(FILE *fp,PolluxBuffer *seqBuffer)
{
  int sta,polSta;
  char line[POLSTRSIZE],field[POLSTRSIZE],trash[POLSTRSIZE];
  
  /* WE COPY THE SEQUENCE IN A BUFFER */
  while(fgets(line,POLSTRSIZE,fp) != NULL)
    {
      if( (sta = sscanf(line,"%s %s",field,trash)) < 1) 
	{ /* THE LINE IS EMPTY: GOTO NEXT LINE */
	  continue;
	}
      else if(sta > 1) 
	/* THE LINE CONTAINS BLANK: ERROR */
	return POLFORMATERROR;

      else if ((polSta = PolLineToBuffer(seqBuffer,field)) != POLOK)
	return polSta;
		  
    }

  return POLOK;
}

/* Idem but to a Long String */
int PolSeqToLongString(FILE *fp,LONGSTRING ls)
{
  int sta;
  char line[POLSTRSIZE],field[POLSTRSIZE],trash[POLSTRSIZE];
  
  LSSetStringEmpty(ls);
  /* WE COPY THE SEQUENCE IN A BUFFER */
  while(fgets(line,POLSTRSIZE,fp) != NULL)
    {
      if( (sta = sscanf(line,"%s %s",field,trash)) < 1) 
	{ /* THE LINE IS EMPTY: GOTO NEXT LINE */
	  continue;
	}
      else if(sta > 1) 
	/* THE LINE CONTAINS BLANK: ERROR */
	return POLFORMATERROR;

      else if (LSConcatLine(ls,field) == NULL)
	return POLOVERFLOW;
		  
    }

  return POLOK;
}

int _PolSeqBufferToFileBuffer(PolluxBuffer *fileBuffer,
			      PolluxBuffer *seqBuffer)
{
  int i,j,lineLengh;
  
  j=0;
  lineLengh = 0;

  for(i = fileBuffer->size ;i < POLBUFFERSIZE-1;i++)
    {
      if(seqBuffer->buffer[j] == '\0')
	{
	  fileBuffer->buffer[i] = '\n';
	  fileBuffer->buffer[i+1] = '\0';
	  fileBuffer->size = i+1;
	  return POLOK;
	}
      
      if(lineLengh == 50)
	{
	  fileBuffer->buffer[i] = '\n';
	  lineLengh = 0;
	}
      else
	{
	  fileBuffer->buffer[i] = seqBuffer->buffer[j];
	  j++;
	  lineLengh++;
	}
    }

  return POLOVERFLOW;
}


int PolSeqBufferToFile(const char *fileName,PolluxBuffer *seqBuffer)
{
  PolluxBuffer fileBuffer;
  
  int polSta; 
  FILE *fpFile; 
  
  /* OPENING FILE fileName */ 
  if((fpFile = fopen(fileName,"r")) == NULL)
    return POLFILEIOERROR;
  
  /* INITIALISATION  */
  fileBuffer.buffer[0] = '\0'; 
  fileBuffer.size = 0; 
  /* LET'S COPY UP TO THE SEQUENCE */
  if((polSta = _PolDefsToBuffer(fpFile,&fileBuffer)) != POLOK)  
    { 
      fclose(fpFile); 
      return polSta; 
    } 
  /*WE ARE FINISHED WITH THE FILE */
  fclose(fpFile); 
  
  
  /* LET'S PUT seqBuffer AT THE END OF THE REST OF THE FILE */
  if((polSta = _PolSeqBufferToFileBuffer(&fileBuffer,seqBuffer) != POLOK)) 
    { 
      fclose(fpFile); 
      return polSta; 
    } 

  /* LET'S WRITE THE RESULT TO THE FILE fileName */
  if((polSta = PolBufferToFile(fileName,&fileBuffer) != POLOK)) 
    { 
      fclose(fpFile); 
      return polSta; 
    } 
  
  return POLOK; 
} 

/***********************************************************************/
/* IT REPLACES FILE fileName WITH THE BUFFER fileBuffer. */
/* IT SAVES A COPY OF THE ORIGINAL FILE UNDER THE NAME fileName~. */
/***********************************************************************/

int PolBufferToFile(const char *fileName,PolluxBuffer *fileBuffer)
{
  char fileNameTemp[POLSTRSIZE];
  int polSta;
  FILE *fp;

  if((polSta =  PolStrCpy(fileNameTemp,fileName) != POLOK))
    return polSta;
  

  /* IF fileName EXISTS WE MAKE A COPY IN fileName~ */
  if( (fp = fopen(fileName,"r")) != NULL)
    {
      fclose(fp);
      if((polSta =  PolStrCat(fileNameTemp,"~") != POLOK))
	return polSta;

      if(rename(fileName,fileNameTemp) != 0)
	{
	  Printf("Can't rename %s in %s !!\n",fileName,fileNameTemp);
	  return POLFILEIOERROR;
	}
    }
  if( (fp = fopen(fileName,"w")) == NULL) 
    return POLFILEIOERROR;

  if (fprintf(fp,"%s",fileBuffer->buffer) < 1)
    {
      fclose(fp);
      return  POLFILEIOERROR;
    }

  fclose(fp);  
  return POLOK;
}

/****************************************************************************/
/* READ A SEQUENCE FROM FILE fileName and puts IN A (SEQ)BUFFER             */
/****************************************************************************/
/* THE BUFFER IS SUPPOSE TO BE A PolBuffer (size = POLBUFFERSIZE)           */
/****************************************************************************/
int PolSequenceToBuffer( const char *fileName, PolluxBuffer *seqBuffer)
{
  int polSta;
  FILE *fpFile;

  if ((fpFile = fopen(fileName,"r")) == NULL)
    return POLFILEIOERROR;

  /* FIRST WE MUST FIND THE SEQUENCE */
  _PolJumpToSequence(fpFile);
  
  /* INITIALISATION */
  seqBuffer->buffer[0] = '\0';
  seqBuffer->size = 0;
  /* WE COPY THE SEQUENCE IN A BUFFER */
  if ((polSta = PolSeqToBuffer(fpFile,seqBuffer)) != POLOK)
    {
      fclose(fpFile);
      return polSta;
    }

  if(seqBuffer->size == 0) 
    { /* NOTHING WAS COPIED TO THE BUFFER */
      fclose(fpFile);
      return POLFORMATERROR;
    }
  
  fclose(fpFile);
  return POLOK;
}

/****************************************************************************/
/* UNDERSAMPLES THE SEQUENCE IN buffer CONSIDERING THAT THE FIRST    */
/* COMPLETE CODON IS IN POSITION phase AND THAT THE RESULTING SEQUENCE      */
/* SHOULD BE A SEQUENCE OF BASES IN POSITION phaseSubSeq IN ITS CODON       */
/* */
/****************************************************************************/
/* */
/* */
/****************************************************************************/
int PolSubSampleBuffer(PolluxBuffer *seqBuffer,int phase,int phaseSubSeq)
{
  int phaseFirstBase,indexFirstToKeep;
  int sizeb,i,j;
  
  if(phaseSubSeq != 1 && phaseSubSeq != 2 && phaseSubSeq != 3)
    return POLFORMATERROR;
  
  if(phase == 1 ) phaseFirstBase = 1;
  else if(phase == 2 ) phaseFirstBase = 3;
  else if(phase == 3 ) phaseFirstBase = 2;
  else return POLFORMATERROR;

  indexFirstToKeep = phaseSubSeq-phaseFirstBase;
  if(indexFirstToKeep < 0) indexFirstToKeep += 3;
  
  sizeb = seqBuffer->size;

  for(i=indexFirstToKeep,j=0;i<sizeb;i+=3,j++)
    seqBuffer->buffer[j] = seqBuffer->buffer[i];
  
  seqBuffer->buffer[j] = '\0';
  seqBuffer->size = j;

  return POLOK;
}

/****************************************************************************/
/* UNDERSAMPLES THE SEQUENCE IN FILE fileName CONSIDERING THAT THE FIRST    */
/* COMPLETE CODON IS IN POSITION phase AND THAT THE RESULTING SEQUENCE      */
/* SHOULD BE A SEQUENCE OF BASES IN POSITION phaseSubSeq IN ITS CODON       */
/* */
/****************************************************************************/
/* */
/* */
/****************************************************************************/
int PolSubSampleSequence(const char *fileNameIn,const char *fileNameOut,int phaseSubSeq)
{
  PolluxBuffer fileBuffer,seqBuffer;
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  FILE *fpFile;
  int polSta;
  long phase;
  char *test;

  PolInitList(fieldList);
  PolInitList(valueList);
  strcpy(fieldList[0],"SUBSAMPLE");
  strcpy(fieldList[1],"PHASE");
  
  /* ARE WE ALLOWED TO SUBSAMPLE ? */
  if((polSta = PolRead(fileNameIn,fieldList,valueList)) != POLOK)
    return polSta;
  if(valueList[0][0] != '\0')
    {
      Printf("The file %s has already been subsample!!\n",fileNameIn);
      return POLFORMATERROR;
    }
  if(valueList[1][0] == '\0')
    {
      Printf("The field PHASE is not defined !!\n");
      return POLFORMATERROR;
    }
  phase = strtol(valueList[1],&test,10);
  if(test == valueList[1] || test != (valueList[1]+strlen(valueList[1])))
    {
      Printf("The field PHASE is not correct !!\n");
      return POLFORMATERROR;
    }
  

  /* OPENING FILE fileNameIn */
  if((fpFile = fopen(fileNameIn,"r")) == NULL)
    {
      Printf("Can't oen file %s",fileNameIn);
      return POLFILEIOERROR;
    }
  
  /* INITIALISATION */
  fileBuffer.buffer[0] = '\0';
  fileBuffer.size = 0;
  /* LET'S COPY UP TO THE SEQUENCE */
  if((polSta = _PolDefsToBuffer(fpFile,&fileBuffer)) != POLOK) 
    {
      fclose(fpFile);
      return polSta;
    }
  
  /* INITIALISATION */
  seqBuffer.buffer[0] = '\0';
  seqBuffer.size = 0;
  /* LET'S PUT THE SEQUENCE IN A BUFFER */
  if((polSta = PolSeqToBuffer(fpFile,&seqBuffer)) != POLOK) 
    {
      fclose(fpFile);
      return polSta;
    }
  
  /*WE ARE FINISHED WITH THE FILE */
  fclose(fpFile);
  
  /* LET'S SUBSAMPLE THE SEQUENCE */
  if((polSta = PolSubSampleBuffer(&seqBuffer,(int) phase,phaseSubSeq)) != POLOK)
    return polSta;
  
  /* LET'S PUT IT BACK AT THE END OF THE REST OF THE FILE */
  if((polSta = _PolSeqBufferToFileBuffer(&fileBuffer,&seqBuffer)) != POLOK)
    return polSta;

  /* LET'S PUT THE RESULTIN FILE fileNameOut */
  if((polSta = PolBufferToFile(fileNameOut,&fileBuffer)) != POLOK)
    {
      Printf("1\n");
      return polSta;
    }

  /* LET'S SAY WHAT WE HAVE DONE */
  fieldList[1][0] = '\0';
  if((polSta = PolAddField(fileNameOut,fieldList)) != POLOK)
    return polSta;
  sprintf(valueList[0],"%ld %s",phase,fileNameIn);
  if((polSta = PolWrite(fileNameOut,fieldList,valueList)) != POLOK)
    return polSta;

  return POLOK;
}


/****************************************************************************/
/* COMPUTE THE SEQUENCE CORRESPONDING TO THE OPPOSITE STRAND */
/* 1) REVERSE THE DIRECTION OF THE SEQUENCE */
/* 2) REPLACE EACH BASE BY ITS COMPLEMENTARY */
/****************************************************************************/
/* */
/* */
/****************************************************************************/
int PolOppStrandBuffer(PolluxBuffer *seqBuffer)
{
  int sizeb,i;
  char c;

  sizeb = seqBuffer->size;
  
  /* FIRST WE INVERSE THE SEQUENCE DIRECTION */
  for(i=0;i<sizeb/2;i++)
    {
      c = seqBuffer->buffer[i];
      seqBuffer->buffer[i] = seqBuffer->buffer[sizeb-i-1];
      seqBuffer->buffer[sizeb-i-1] = c;
    }
  
  /* THEN WE REPLACE EACH BASE BY ITS COMPLEMENTARY */
  for(i=0;i<sizeb;i++)
    {
      switch(seqBuffer->buffer[i])
	{
	case 'A':
	case 'a':
	  seqBuffer->buffer[i] = 't';
	  break;
	case 'T':
	case 't':
	  seqBuffer->buffer[i] = 'a';
	  break;
	case 'G':
	case 'g':
	  seqBuffer->buffer[i] = 'c';
	  break;
	case 'C':
	case 'c':
	  seqBuffer->buffer[i] = 'g';
	  break;
	case 'R':
	case 'r':
	  seqBuffer->buffer[i] = 'y';
	  break;
	case 'Y':
	case 'y':
	  seqBuffer->buffer[i]  = 'r';
	  break;
	case 'S':
	case 's':
	  seqBuffer->buffer[i] = 's';
	  break;
	case 'W':
	case 'w':
	  seqBuffer->buffer[i] = 'w';
	  break;
	case 'M':
	case 'm':
	  seqBuffer->buffer[i] = 'k';
	  break;
	case 'K':
	case 'k':
	  seqBuffer->buffer[i] = 'm';
	  break;
	case 'B':
	case 'b':
	  seqBuffer->buffer[i] = 'v';
	  break;
	case 'H':
	case 'h':
	  seqBuffer->buffer[i] = 'd';
	  break;
	case 'D':
	case 'd':
	  seqBuffer->buffer[i] = 'h';
	  break;
	case 'V':
	case 'v':
	  seqBuffer->buffer[i] = 'b';
	  break;
	case 'N':
	case 'n':
	  seqBuffer->buffer[i] = 'n';
	  break;
	default:
	  return POLFORMATERROR;
	  break;
	}
    }
  
  return POLOK;
}

/****************************************************************************/
/* COMPUTE THE SEQUENCE CORRESPONDING TO THE OPPOSITE STRAND */
/* 1) REVERSE THE DIRECTION OF THE SEQUENCE */
/* 2) REPLACE EACH BASE BY ITS COMPLEMENTARY */
/****************************************************************************/
/* */
/* */
/****************************************************************************/
int PolOppStrandSequence(const char *fileName)
{
  PolluxBuffer fileBuffer,seqBuffer;
  char fieldList[POLSIZELIST][POLSTRSIZE],valueList[POLSIZELIST][POLSTRSIZE];
  FILE *fpFile;
  int polSta;

  /* OPENING FILE fileName */
  if((fpFile = fopen(fileName,"r")) == NULL)
    return POLFILEIOERROR;

  /* INITIALISATION */
  fileBuffer.buffer[0] = '\0';
  fileBuffer.size = 0;
  /* LET'S COPY UP TO THE SEQUENCE */
  if((polSta = _PolDefsToBuffer(fpFile,&fileBuffer)) != POLOK) 
    {
      fclose(fpFile);
      return polSta;
    }

  /* INITIALISATION */
  seqBuffer.buffer[0] = '\0';
  seqBuffer.size = 0;
  /* LET'S PUT THE SEQUENCE IN A BUFFER */
  if((polSta = PolSeqToBuffer(fpFile,&seqBuffer)) != POLOK) 
    {
      fclose(fpFile);
      return polSta;
    }

  /*WE ARE FINISHED WITH THE FILE */
  fclose(fpFile);
  
  /* LET'S COMPUTE THE OPPOSITE STRAND SEQUENCE*/
  if((polSta = PolOppStrandBuffer(&seqBuffer)) != POLOK)
    return polSta;

  /* LET'S PUT IT BACK AT THE END OF THE REST OF THE FILE */
  if((polSta = _PolSeqBufferToFileBuffer(&fileBuffer,&seqBuffer)) != POLOK)
    return polSta;



  /* LET'S REPLACE FILE fileName WITH fileBuffer */
  if((polSta = PolBufferToFile(fileName,&fileBuffer)) != POLOK)
    return polSta;

  /* LET'S SAY WHAT WE HAVE DONE */
  PolInitList(fieldList);
  PolInitList(valueList);
  strcpy(fieldList[0],"REVERSED");
  /* DID THE FIELD REVERSED ALREADY EXIST ?*/
  if((polSta = PolRead(fileName,fieldList,valueList)) != POLOK)
    return polSta;
  /* IF YES REMOVE IT */
  if(valueList[0][0] != '\0')
    {
      if((polSta = PolRemoveField(fileName,fieldList)) != POLOK)
	return polSta;
    }
  /* IF NO ADD IT */
  else
    if((polSta = PolAddField(fileName,fieldList)) != POLOK)
      return polSta;


  return POLOK;
}
