/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2005.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: Benjamin.Audit@ens-lyon.fr                                 */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

/****************************************************************************/
/*                                                                          */
/*                   STATISTICS ON SEQUENCES                                */
/*                                                                          */
/****************************************************************************/
#include "pollux.h"

/****************************************************************************/
/* COMPUTES THE DEGREE OF LIKENESS OF TWO SEQUENCES                         */
/* IT COUNTS THE NUMBER OF WORDS OF LENGH n OF SEQUENCE A (IN bufferA) THAT */
/* ALSO EXIST IN SEQUENCE B (IN bufferB)                                    */
/****************************************************************************/
/* n MUST BE SMALLER THAN POLSTRSIZE - 1                                    */
/****************************************************************************/
FLOAT PolSimilitude(const char *polbufferA,const char *polbufferB, int n)
{
  int indexA,indexB,sizeA,sizeB,count;

  count = 0;
  sizeA = strlen(polbufferA);
  sizeB = strlen(polbufferB);

  /* LOOP ON ALL THE WORD OF LENGH n OF bufferA */
  for(indexA=0;indexA <= sizeA - n; indexA++)
    {
      /* LOOP ON ALL THE WORD OF LENGH n OF bufferB */
      for(indexB=0;indexB <= sizeB - n; indexB++)
	{
	  /* IF THE A-WORD IS FOUND ONCE IN B GOTO NEXT A-WORD */
	  if( strncasecmp((polbufferA+indexA),(polbufferB+indexB),n) == 0)
	    {
	      count++;
	      break;
	    }
	}
    }
  return (FLOAT) count / ((FLOAT)(sizeA - n + 1));
}

