/*..........................................................................*/
/*                                                                          */
/*      Pollux 2.1                                                          */
/*                                                                          */
/*      Authors: Benjamin Audit, Emmanuel Bacry and Cedric Vaillant         */
/*      Copyright (C) 1997-2002.  All Right Reserved.                       */
/*                                                                          */
/*      Contact: enjamin.Audit@ens-lyon.fr                                  */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/

/****************************************************************************/
/*                                                                          */
/*               WRITE POLLUX FILES                                         */
/*                                                                          */
/****************************************************************************/



#include "pollux.h"
#include <stdlib.h>


/****************************************************************************/
/* EXTRACT ALL fieldS IN fieldList (**char) READ THE CORRESPONDING          */
/* values IN valueList (**char) AND COPY THEM IN THE FILE fileName          */
/****************************************************************************/
/* THIS PROCEDURE DOESN'T CREATE NEW field BUT IT TELLS WHEN YOU WANT TO    */
/* CHANGE THE VALUE OF A NON EXISTING field                                 */
/****************************************************************************/
int PolWrite(const char *fileName,char fieldList[][POLSTRSIZE],char valueList[][POLSTRSIZE])
{
  PolluxBuffer buffer;
  char line[POLSTRSIZE],field[POLSTRSIZE],command[POLSTRSIZE+1];
  char c;
  FILE *fpFile;
  int flagCopied[POLSIZELIST],polSta,i;
  
  if ((fpFile = fopen(fileName,"r")) == NULL)
    {
      Printf("can't open file %s\n",fileName);
      return POLFILEIOERROR;
    }

  /* INITIALISATION */
  buffer.buffer[0] = '\0';
  buffer.size = 0;
  for(i=0;i<POLSIZELIST;i++)  flagCopied[i] = 0;


  /* THE FILE WILL BE TREATED LINE BY LINE */
  while(fgets(line,POLSTRSIZE,fpFile)!=NULL)
    {
      if (sscanf(line,"%1s",&c) < 1)
	{
	  /* THE LINE IS EMPTY: SO THE LINE IS NOT COPIED */
	  continue;
	}
      switch(c)
	{
	case '=':
	  if (sscanf(line," =%s ",field) < 1)
	    {
	      /* THEIR IS NO FIELD: SO THE LINE IS NOT COPIED */
	      continue;
	    }
	  /* FROM THAT POINT FIELD IS WELL DEFINED */
	  if (!strcmp(field,"SEQUENCE")) 
	    {
	      /* THE SEQUENCE ZONE STARTS HERE: WE COPY THE REST OF THE FILE */
	      /* WE DO NOT COPY THE NEW value CORRESPONDING TO THIS FIELD */
	      do 
		{
		  if((polSta = PolLineToBuffer(&buffer,line)) != POLOK) 
		    {
		      fclose(fpFile);
		      return polSta;
		    }
		}
	      while(fgets(line,POLSTRSIZE,fpFile)!=NULL);
	    }
	  else
	    {

	      /* IF FIELD IS PART OF THE FIELD LIST */
	      /* THEN LINE IS MODIFIED TO FIELD-NEWVALUE */
	      if(IsOfStringList(field,fieldList,&i) == POLYES)
		{
		  if((polSta = PolFieldValueToLine(line,field,valueList[i])) != POLOK)
		    {
		      fclose(fpFile);
		      return polSta;
		    }
		  /* AND flagCopied IS SET TO 1 TO SAY fieldList[i] WAS FOUND AND ITS VALUE COPIED */
		  flagCopied[i] = 1;		  
		}
	    

	      /* THE LINE MODIFIED OR NOT IS COPIED */
	      if((polSta = PolLineToBuffer(&buffer,line)) != POLOK) 
		{
		  fclose(fpFile);
		  return polSta;
		}
	      
	    }
	  break;
	case '%':
	  /* IT IS A COMMENT LINE: THE LINE IS COPIED */
	  if((polSta = PolLineToBuffer(&buffer,line)) != POLOK) 
	    {
	      fclose(fpFile);
	      return polSta;
	    }
	  break;
	default:
	  fclose(fpFile);
	  return POLFORMATERROR;
	}
    }
  fclose(fpFile);
  
  sprintf(command,"mv %s %s~",fileName,fileName);
  if (system(command) == -1) return POLFILEIOERROR;
  if ((fpFile = fopen(fileName,"w")) == NULL) return POLFILEIOERROR;
  if (fprintf(fpFile,"%s",buffer.buffer) < 1) {
    fclose(fpFile);
    return  POLFILEIOERROR;
  }
  fclose(fpFile);  

  for(i=0;(fieldList[i][0] != '\0') && (i < POLSIZELIST);i++) 
    {
      if (!strcmp(fieldList[i],"SEQUENCE"))
	Printf("Warning: option SEQUENCE is not a valid option (I disregarded it).\n");
      else if(flagCopied[i] == 0) 
	Printf("Can't change the value of the non-existing field:%s in file:%s\n",fieldList[i],fileName);
    }
  return POLOK;
}

/****************************************************************************/
/* ADD NEW FIELDS FROM A fieldList TO A POLLUX FILE                         */
/* IF ONE OF THE FIELD ALREADY EXISTS, IT JUST PROMPTS IT                   */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/ 
int PolAddField(const char *fileName, char fieldList[][POLSTRSIZE])
{
  PolluxBuffer buffer;
  char line[POLSTRSIZE],field[POLSTRSIZE],tempLine[POLSTRSIZE];
  char c;
  FILE *fpFile;
  int flagExist[POLSIZELIST],polSta,i,flagCopied=0;
  
  if ((fpFile = fopen(fileName,"r")) == NULL)
    {
      Printf("can't open file %s\n",fileName);
      return POLFILEIOERROR;
    }

  /* INITIALISATION */
  buffer.buffer[0] = '\0';
  buffer.size = 0;
  for(i=0;i<POLSIZELIST;i++)  flagExist[i] = 0;

  /* WE HAVE TO MAKE SURE THAT THE SPECIAL FIELD SEQUENCE IS NOT ADDED */
  if(IsOfStringList("SEQUENCE",fieldList,&i) == POLYES) flagExist[i] = 1;

  /* THE FILE WILL BE TREATED LINE BY LINE */
  while(fgets(line,POLSTRSIZE,fpFile)!=NULL)
    {
      if (sscanf(line,"%1s",&c) < 1)
	{
	  /* THE LINE IS EMPTY: SO THE LINE IS NOT COPIED */
	  continue;
	}
      switch(c)
	{
	case '=':
	  if (sscanf(line," =%s ",field) < 1)
	    {
	      /* THEIR IS NO FIELD: SO THE LINE IS NOT COPIED */
	      continue;
	    }
	  if (!strcmp(field,"SEQUENCE")) 
	    {
	      flagCopied = 1;
	      /* THE SEQUENCE ZONE STARTS HERE: WE ADD THE NEW FIELDS */
	      for(i=0;(i < POLSIZELIST) && (fieldList[i][0] != '\0');i++)
		{
		  if(flagExist[i] == 0)
		    {
		      sprintf(tempLine,"=%s\n",fieldList[i]);
		      if ((polSta = PolLineToBuffer(&buffer,tempLine)) != POLOK)
			{
			  fclose(fpFile);
			  return polSta;
			}
		    }
		}
	      /* THEN COPY THE REST OF THE FILE */
	      do 
		{
		  if ((polSta = PolLineToBuffer(&buffer,line)) != POLOK)
		    {
		      fclose(fpFile);
		      return polSta;
		    }
		}
	      while(fgets(line,POLSTRSIZE,fpFile)!=NULL);
	    }
	  else
	    {
	      /* IF FIELD IS PART OF THE FIELD LIST: WE JUST SET flagExist[i] TO 1 */
	      if(IsOfStringList(field,fieldList,&i) == POLYES) flagExist[i] = 1;
		 
	      /* THE LINE IS COPIED */
	      if ((polSta = PolLineToBuffer(&buffer,line)) != POLOK)
		{
		  fclose(fpFile);
		  return polSta;
		}

	    }
	  break;
	case '%':
	  /* IT IS A COMMENT LINE: THE LINE IS COPIED */
	  if ((polSta = PolLineToBuffer(&buffer,line)) != POLOK)
	    {
	      fclose(fpFile);
	      return polSta;
	    }
	  break;
	default:
	  fclose(fpFile);
	  return POLFORMATERROR;
	}
    }
  if(flagCopied == 0)
    /* NO SEQUENCE FIELD WERE FOUND SO WE WE HAVE TO ADD THE NEW FIELD NOW */
    {
      for(i=0;(i < POLSIZELIST) && (fieldList[i][0] != '\0');i++)
	{
	  if(flagExist[i] == 0)
	    {
	      sprintf(tempLine,"=%s\n",fieldList[i]);
	      if ((polSta = PolLineToBuffer(&buffer,tempLine)) != POLOK)
		{
		  fclose(fpFile);
		  return polSta;
		}
	    }
	}
    }
  fclose(fpFile);
  
  sprintf(tempLine,"mv %s %s~",fileName,fileName);
  if (system(tempLine) == -1) return POLFILEIOERROR;
  if ((fpFile = fopen(fileName,"w")) == NULL) return POLFILEIOERROR;
  if (fprintf(fpFile,"%s",buffer.buffer) != buffer.size) {
    fclose(fpFile);
    return  POLFILEIOERROR;
  }
  fclose(fpFile);  
  
  for(i=0;(i < POLSIZELIST) && (fieldList[i][0] != '\0');i++)
    {
      if (!strcmp(fieldList[i],"SEQUENCE"))
	Printf("Warning: option SEQUENCE is not a valid option (I disregarded it).\n");
      else if(flagExist[i] == 1)
	Printf("Field: %s already existed\n",fieldList[i]);
    }
  return POLOK;
}


/****************************************************************************/
/* REMOVE FIELDS OF A fieldList FROM A POLLUX FILE                         */
/* IF ONE OF THE FIELD DIDN'T EXIST, IT JUST PROMPTS IT                   */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/****************************************************************************/ 
int PolRemoveField(const char *fileName, char fieldList[][POLSTRSIZE])
{
  PolluxBuffer buffer;
  char line[POLSTRSIZE],field[POLSTRSIZE],tempLine[POLSTRSIZE];
  char c;
  FILE *fpFile;
  int flagExist[POLSIZELIST],polSta,i;
  
  if ((fpFile = fopen(fileName,"r")) == NULL)
    {
      Printf("can't open file %s\n",fileName);
      return POLFILEIOERROR;
    }

  /* INITIALISATION */
  buffer.buffer[0] = '\0';
  buffer.size= 0;
  for(i=0;i<POLSIZELIST;i++)  flagExist[i] = 0;


  /* THE FILE WILL BE TREATED LINE BY LINE */
  while(fgets(line,POLSTRSIZE,fpFile)!=NULL)
    {
      if (sscanf(line,"%1s",&c) < 1)
	{
	  /* THE LINE IS EMPTY: SO THE LINE IS NOT COPIED */
	  continue;
	}
      switch(c)
	{
	case '=':
	  if (sscanf(line," =%s ",field) < 1)
	    {
	      /* THEIR IS NO FIELD: SO THE LINE IS NOT COPIED */
	      continue;
	    }
	  if (!strcmp(field,"SEQUENCE")) 
	    {
	      /* THE SEQUENCE ZONE STARTS HERE: WE COPY THE REST OF THE FILE */
	      do 
		{
		  if ((polSta = PolLineToBuffer(&buffer,line)) != POLOK)
		    {
		      fclose(fpFile);
		      return polSta;
		    }
		}
	      while(fgets(line,POLSTRSIZE,fpFile)!=NULL);
	    }
	  
	  /* IF FIELD IS PART OF THE FIELD LIST: WE JUST SET flagExist[i] TO 1 */
	  /* BUT WE DON'T COPY THE LINE */
	  else if(IsOfStringList(field,fieldList,&i) == POLYES) flagExist[i] = 1;
	    
		 
	  else /* THE LINE IS COPIED */
	    {
	      if ((polSta = PolLineToBuffer(&buffer,line)) != POLOK)
		{
		  fclose(fpFile);
		  return polSta;
		}
	      
	    }
	  break;
	case '%':
	  /* IT IS A COMMENT LINE: THE LINE IS COPIED */
	  if ((polSta = PolLineToBuffer(&buffer,line)) != POLOK)
	    {
	      fclose(fpFile);
	      return polSta;
	    }
	  break;
	default:
	  fclose(fpFile);
	  return POLFORMATERROR;
	  break;
	}
    }
  fclose(fpFile);
  
  sprintf(tempLine,"mv %s %s~",fileName,fileName);
  if (system(tempLine) == -1) return POLFILEIOERROR;
  if ((fpFile = fopen(fileName,"w")) == NULL) return POLFILEIOERROR;
  if (fprintf(fpFile,"%s",buffer.buffer) != buffer.size) 
    {
      fclose(fpFile);
      return  POLFILEIOERROR;
    }
  fclose(fpFile);  
  
  for(i=0;(fieldList[i][0] != '\0') && (i < POLSIZELIST);i++)
    {
      if (!strcmp(fieldList[i],"SEQUENCE"))
	Printf("Warning: option SEQUENCE is not a valid option (I disregarded it).\n");
      else if(flagExist[i] == 0)
	Printf("Field: %s didn't exist\n",fieldList[i]);
    }
  return POLOK;
}










