#include "lastwave.h"
#include "wtrans1d.h"
#include "images.h"
#include "stft.h"

static void _Gregfile_Write_One_Float(FILE *fp,char *format,FLOAT theval) {
  float myfloat4;
  unsigned int myint4; /* in hope this is a 32bit (4octet) integer and that flaot are also 32bit */

  if( format != NULL ) {
    fprintf(fp,format,theval);
  } else {
    myfloat4 = (float) theval;
    myint4 = *(int *)& myfloat4;
    fprintf(fp,"%08x\n",myint4);
  }
}

static void _Gregfile_Write_Wtrans(WTRANS wtrans,FILE *fp,FLOAT q,char *format) {
  int size,noct,nvoi,i,o,v;
  FLOAT x0,dx,ln2amin;

  size = wtrans->size;
  noct = wtrans->nOct;
  nvoi = wtrans->nVoice;
  x0  = wtrans->x0;
  dx  = wtrans->dx;
  ln2amin = log(wtrans->aMin)/log(2.);

  
  /* greg header: size ref val inc 
     with: X(i)=(i-ref)*inc+val
     */
  fprintf(fp,"%d 1 %f %f\nhaha\n%d 1 %f %f\nhoho\n",
	  size,x0,dx,noct*nvoi,ln2amin,1./nvoi);
  
  for (o = 1; o <= noct; o++) {
    for (v = 0; v < nvoi; v++) {
      if (wtrans->D[o][v]->size != size)
	Errorf("Error in _Gregfile_Write_Wtrans(): "
	       "wtrans->D[o][v]->size should match wtrans->size.");
      if(q<=0) {
	for (i = 0; i < size; i++) 
	  /* fprintf(fp,format,wtrans->D[o][v]->Y[i]); */
	  _Gregfile_Write_One_Float(fp,format,wtrans->D[o][v]->Y[i]);
      } else {
	for (i = 0; i < size; i++) 
	  /* fprintf(fp,format,pow(fabs(wtrans->D[o][v]->Y[i]),q)); */
	  _Gregfile_Write_One_Float(fp,format,pow(fabs(wtrans->D[o][v]->Y[i]),q));
      }
    }
  }

}

static void _Gregfile_Write_Image(IMAGE image, FILE *fp, FLOAT q, char *format,
				  FLOAT x_x0, FLOAT x_dx, FLOAT y_x0, FLOAT y_dx) {
  int size,ic,ir;

  if( (image->nrow <= 0)
      || (image->ncol <= 0)
      )
    Errorf("Empty image !!");
    
  /* greg header: size ref val inc 
     with: X(i)=(i-ref)*inc+val
  */
  fprintf(fp,"%d 1 %f %f\nhaha\n%d 1 %f %f\nhoho\n",
	  image->ncol,x_x0,x_dx,
	  image->nrow,y_x0,y_dx);
  
  /* row in lw-images are top-bottom but are bottom-top in greg*/
  /* for (ir = image->nrow-1; ir >=0; ir--) {  */
   for (ir = 0; ir < image->nrow; ir++) { 
    for (ic = 0; ic < image->ncol; ic++) {
      if(q<=0) {
	/* fprintf(fp,format,image->pixels[image->ncol*ir+ic]); */
	_Gregfile_Write_One_Float(fp,format,image->pixels[image->ncol*ir+ic]);
      } else {
	/* fprintf(fp,format,pow(fabs(image->pixels[image->ncol*ir+ic]),q)); */
	_Gregfile_Write_One_Float(fp,format,pow(fabs(image->pixels[image->ncol*ir+ic]),q));
      }
    }
  }
  
}

static void _Gregfile_Write_Stft(STFT stft,FILE *fp,FLOAT q,char *format) {
  int sizex,sizey,i,j,index;
  FLOAT x0,dx,y0,dy, norm2;

  sizex = stft->nFrames;
  x0  = stft->x0+((stft->tRate-1)/2.0)*stft->dx;
  dx  = stft->tRate*stft->dx;

  sizey = stft->nSubBands;
  y0  = 0;
  dy  = 0.5/(stft->nSubBands-1);
  
  /* greg header: size ref val inc 
     with: X(i)=(i-ref)*inc+val
     */
  fprintf(fp,"%d 1 %g %g\nhaha\n%d 1 %g %g\nhoho\n",
	  sizex,x0,dx,sizey,y0,dy);
  

  for(j = 0; j < stft->nSubBands; j++) {
    for(i = 0; i < stft->nFrames; i++) {

      index = (i*stft->nSubBands)+j;

      switch(stft->type) {
      case ComplexStft :
	norm2 = stft->real->Y[index]*stft->real->Y[index] + stft->imag->Y[index]*stft->imag->Y[index];
	break;

      case RealStft :
      case HighResStft :
	norm2 = stft->real->Y[index];
	break;
	
      default :
	Errorf("_Gregfile_Write_Stft : unknown type %d",stft->type);
      }
      
      
      if(q<=0) {
	_Gregfile_Write_One_Float(fp,format,norm2);
      } else {
	_Gregfile_Write_One_Float(fp,format,pow(norm2,0.5*q));
      }
    }
  }

}

static void C_Gregfile_Write(char **argv)
{
  WTRANS wtrans = NULL;
  IMAGE image = NULL;
  STFT stft = NULL;
  char *filename;
  char *format;
  FILE *fp;
  char opt;
  FLOAT q;
  FLOAT x_x0, x_dx, y_x0, y_dx;

  argv = ParseArgv(argv,tWTRANS_,NULL,&wtrans,-1);
  if(wtrans == NULL) {
    argv = ParseArgv(argv,tSTFT_,NULL,&stft,-1);
    if(stft == NULL) {
      argv = ParseArgv(argv,tIMAGEI,&image,-1);
    }
  }

  argv = ParseArgv(argv,tSTR,&filename,
		   tFLOAT,&q,
		   tSTR_,NULL,&format,-1); /* "%#.8g\n" */

  if(image != NULL) {
    x_x0 = 0.0; x_dx = 1.0; y_x0 = 0.0; y_dx = 1.0;
      while(opt = ParseOption(&argv)) {
      switch(opt) {
      case 'l':
	argv = ParseArgv(argv, tFLOAT, &x_x0, tFLOAT, &x_dx, tFLOAT, &y_x0, tFLOAT, &y_dx, -1);
	break;
      default: 
	ErrorOption(opt);
      }
    }
  }
  NoMoreArgs(argv);

  if( format == NULL ) {
    if( (sizeof(float)!=4) || (sizeof(int)!=4) )
      Errorf("Unexpected size of float or size of int for hexadecimal writing !");
  }

  fp = FOpen(filename,"w");

  if( wtrans != NULL ) {
    _Gregfile_Write_Wtrans(wtrans,fp,q,format);
  } else if( stft != NULL ) {
    _Gregfile_Write_Stft(stft,fp,q,format);
  } else if( image != NULL ) {
    _Gregfile_Write_Image(image,fp,q,format, x_x0, x_dx, y_x0, y_dx);
  } else {
    Errorf("C_Gregfile_Write : Weird !!!\n");
  }

  FClose(fp);
}

#define GREGLINESIZE 128
static void C_Gregfile_Read(char **argv)
{
  WTRANS wtrans;
  IMAGE image;
  char *filename,*format;
  char line[GREGLINESIZE+1];
  FILE *fp;
  int sizex,sizey,noct,nvoi,i,o,v,ic,ir;
  FLOAT x0,dx,y0,dy;
  
  argv = ParseArgv(argv,tWTRANS_,NULL,&wtrans,-1);
  if(wtrans == NULL) {
    argv = ParseArgv(argv,tIMAGE,&image,-1);
  }
  argv = ParseArgv(argv,tSTR,&filename,0);

  fp = FOpen(filename,"r");  

  if(sizeof(FLOAT) == 4) {
    format = "%d 1 %f %f";
  } else if(sizeof(FLOAT) == 8) {
    format = "%d 1 %lf %lf";
  } else {
    Errorf("Unexpected size of FLOAT");
  }
  /* greg header: size ref val inc 
     with: X(i)=(i-ref)*inc+val
     */

  if(fgets(line,GREGLINESIZE,fp) == NULL)
    Errorf("Reading error in the first line of the header");
  if(sscanf(line,format,&sizex,&x0,&dx) != 3)
    Errorf("Format error in the first line of the header");

  if(fgets(line,GREGLINESIZE,fp) == NULL)
    Errorf("Reading error in the second line of the header");
  
  if(fgets(line,GREGLINESIZE,fp) == NULL)
    Errorf("Reading error in the third line of the header");
  if(sscanf(line,format,&sizey,&y0,&dy) != 3)
    Errorf("Format error in the third line of the header");

  if(fgets(line,GREGLINESIZE,fp) == NULL)
    Errorf("Reading error in the fourth line of the header");

  /* Format for reading one FLOAT */
  if(sizeof(FLOAT) == 4) {
    format = "%f ";
  } else if(sizeof(FLOAT) == 8) {
    format = "%lf ";
  } else {
    Errorf("Unexpected size of FLOAT");
  }

  if(wtrans == NULL) {
    SizeImage(image,sizex,sizey); /* SizeImage does not follow the order line-col */

    /* row in lw-images are top-bottom but are bottom-top in greg */
    /*    for (ir = image->nrow-1; ir >=0; ir--) */ 
    for (ir = 0; ir < image->nrow; ir++) { 
      for (ic = 0; ic < image->ncol; ic++) {
	if(fscanf(fp,format,&(image->pixels[image->ncol*ir+ic])) != 1)
	  Errorf("Error while reading a float (ir=%d,ic=%d)",ir,ic);
      }
    }
    
  } else {

    nvoi = (int) (1./dy+0.5);
    noct = sizey/nvoi;
    if(noct*nvoi != sizey)
      Errorf("Pb with sizey");

    for (o = 1; o <= noct; o++) {
      for (v = 0; v < nvoi; v++) {
	SizeSignal(wtrans->D[o][v],sizex,YSIG);
	wtrans->D[o][v]->x0 = x0;
	wtrans->D[o][v]->dx = dx;
	for (i = 0; i < sizex; i++) {
	  if(fscanf(fp,format,&(wtrans->D[o][v]->Y[i])) != 1)
	    Errorf("Error while reading a float (o=%d,v=%d,i=%d)",o,v,i);
	}
      }
    }

    wtrans->size = sizex;
    wtrans->nOct = noct;
    wtrans->nVoice = nvoi;
    wtrans->x0  = x0;
    wtrans->dx  = dx;
    wtrans->aMin = exp(y0*log(2.));
  }

  FClose(fp);  
}

void C_Greg(char **argv)
{
  char *action;

  argv = ParseArgv(argv,tWORD,&action,-1);
  
  /* 'read' action */
  if (!strcmp(action,"read")) {
    C_Gregfile_Read(argv);

    /* 'write' action */
  } else if (!strcmp(action,"write")) {
    C_Gregfile_Write(argv);

  } else {
    Errorf("Unknow action '%s'",action);
  }
}

static CProc gregCommands[] = {

  "greg",C_Greg,
  "{{write <wtrans>|<image> <filename> <q> [<format>= NULL] [-l x_x0 x_dx y_x0 x_dx]} "
  "{It creates a file <filename> to be read with the GREG command:\n"
  " rgdata <filename> /format fmt [code].\n"
  "Values are writen using format e.g. '%#.8g\\n' can then be read "
  "with /format E14.7 in GREG.\n"
  "When no format is provided, values are written as hexadecimal representation "
  "of a float4 to be read with '/format z8 r4'.\n"
  "Option -l allow to define pixels coordinates (for images only).}} "
  "{{read <wtrans> <filename>} "
  "{It reads a file <filename> ceated by 'greg write'}} ",

  NULL,NULL,NULL
};

CProcTable gregTable = 
{gregCommands, "greg",
 "Commands to somehow interface LastWave to Greg"};
