#include "lastwave.h"
#include "signals.h"
#include <math.h>
#define BETA 1      /*Temperature*/
#define LAMBDA 1 /*0.398942 normalisation 1/sqrt(2*pi)*/
#define max(a,b) (a>=b?a:b)
#define min(a,b) (a<=b?a:b)



void Vanderlick(SIGNAL pot, double sigma, double mu)
{
  int i,j,n, L,k,m,max,ind;
  double Somme(SIGNAL s, int a, int b);
  double Q,pourcent,S_0,S_1,S_2,S_3,S_1_new;
    
	
  /*Calcul des parametres*/
  L= pot->size;
  n = (int) L/sigma;
  SIGNAL h=NewSignal();
  SIGNAL g=NewSignal();
  SIGNAL r=NewSignal();


  SizeSignal(h,L,YSIG);
  ZeroSig(h);
  SizeSignal(g,L,YSIG);
  ZeroSig(g);
  SizeSignal(r,L,YSIG);
  ZeroSig(r);

  L--; /*L=L-1 so that it makes things clearer*/
	
  /*First step: evaluating h_0*/
  S_0=0;
    
  for (k=1; k<=sigma;k++)
    {
      h->Y[k] =exp(-BETA*(pot->Y[k]))/(exp(-BETA*mu)+S_0);
      S_0=S_0+(exp(-BETA*pot->Y[k])+exp(-BETA*pot->Y[k+1]))/2;
    }

  /*Second step evaluating the rest of h*/
  for (j=1; j <n;j++)
    {

      S_1_new=(h->Y[(j-1)*(int) sigma]+h->Y[(j-1)*(int) sigma+1])/2;
      S_1=0;
      S_2=(exp(S_1-BETA*pot->Y[j*(int)sigma])+exp(S_1_new-BETA*pot->Y[j*(int) sigma+1]))/2;
      S_1=S_1_new;

      for (k=(int) j*sigma+1;k<=min((j+1)*sigma,L-1); k++)
        {
	  h->Y[k] = exp(-BETA*pot->Y[k]+S_1);
	  h->Y[k]/= exp(-BETA*pot->Y[(int) (j*sigma)])/(h->Y[(int) (j*sigma)])+S_2;
			
	  S_1_new=S_1+(h->Y[k-(int) sigma]+h->Y[k-(int) sigma+1])/2;
	  S_2+=(exp(S_1_new-BETA*pot->Y[k+1])+exp(S_1-BETA*pot->Y[k]))/2;
	  S_1=S_1_new;
        }

    }
  /*The last index has to be treated a little bit differently or
   * else there might be out of domain problems*/
  j--;/*The index j has been incremented before the test j<n!!*/
  h->Y[L]=exp(-BETA*pot->Y[L]+S_1);
  h->Y[L]/=exp(-BETA*pot->Y[(int) (j*sigma)])/(h->Y[(int) (j*sigma)])+S_2;




  /*Et zou on repart dans l'autre sens pour calculer l*/
  /*First step: evaluating g_0*/
  S_0=0;
  for (k=L-1; k >=L-sigma; k--)
    {
      g->Y[k]=exp(-S_0);
      S_0+=0.5*(h->Y[k-1]+h->Y[k]);
    }
    
  /*Second step rest of g*/
  for (j=1; j <n;j++)
    {
      /*Une petite variable en plus qui fait pas mal au seins*/
      ind=L-j*(int) sigma;
      S_1=0.5*(h->Y[ind]+h->Y[ind-1]);
      S_2=0.5*(g->Y[ind+(int) sigma]*h->Y[ind+(int) sigma]*exp(-S_1)+g->Y[ind-1+(int) sigma]*h->Y[ind-1+(int) sigma]);
        
		
		
      for (k=(int) (L-j*sigma-1); k >= max(L-(j+1)*sigma,1); k--)
        {

	  g->Y[k]=g->Y[(int) (L-j*sigma)]*exp(-S_1);
	  g->Y[k]+=S_2;

	  S_1+=0.5*(h->Y[k]+h->Y[k-1]);
	  S_3= 0.5*(h->Y[k]+h->Y[k-1]);
	  S_2=exp(-S_3)*S_2+0.5*(g->Y[k+(int) sigma]*h->Y[k+(int) sigma]*exp(-S_3)+g->Y[k-1+(int) sigma]*h->Y[k-1+(int) sigma]);

        }
    }
  /*Again there need to be some specification for the last step*/
  j--;
  g->Y[0]=g->Y[(int) (L-j*sigma)]*exp(-S_1)+S_2;

	
  /*The very last thing: combining g and h*/
  for (k=0; k <= L;k++)
    {
      r->Y[k]=g->Y[k]*h->Y[k];
    }
  SetResultValue(r);
  DeleteSignal(h);
  DeleteSignal(g);
  DeleteSignal(r);
}










/*Commnande C intermedaire pour l'interprtation des arguments de la fonciton
 * vand*/
void C_vand(char **argv)
{
  SIGNAL pot;
  double sigma,mu;
  argv = ParseArgv(argv,tSIGNALI,&pot,tDOUBLE,&sigma,tDOUBLE,&mu,0);
  if (sigma<50) {
    Printf("WARNING: sigma is too small, use bigger sigma to prevent drifts in integration\n");		
  }

  if (pot->size>sigma*2000) {
    Printf("WARNING: Drifts in integration can no longer be neglected.\nUse smaller size potential to prevent this problem.\n");
  }
  Vanderlick(pot,sigma,mu);
}










void turnY(SIGNAL sig_xy)
{
  int i;
  SIGNAL sigY=NewSignal();
  SizeSignal(sigY,sig_xy->size,YSIG);

  for (i=0;i<sig_xy->size;i++)
    {
      sigY->Y[i]=sig_xy->Y[i];
    }
  SetResultValue(sigY);
  DeleteSignal(sigY);
}

void C_turnY(char **argv)
{
  SIGNAL signal_xy;

  argv = ParseArgv(argv,tSIGNALI,&signal_xy,0);
  turnY(signal_xy);
}







/*Set C_command interpretation parameters for LastWave*/
static CProc vanProcs[] = {
  "vand",C_vand,"{{{<pot><sigma><mu>} {Main procedure Vanderlick}}}",
  "turnY",C_turnY,"{{{ <signal_xy>} {turns xy into Ysig} }}",
  NULL,NULL,NULL
};

CProcTable vanderlickTable = {vanProcs, "Vanderlick", "Some procs for Vanderlick."};





























/**********************************************************************************************/
/*Procedure de Cedric*/


int Increment(SIGNAL signalin1,SIGNAL signalout,int l)
{
  int size,i;

  size = signalin1->size;


  SizeSignal(signalout,size-l-1,YSIG);
  signalout->dx = 1;
  signalout->x0 = 0;
  ZeroSig(signalout);
  signalout->Y[0] = (double) 0;

  for(i=0;i<l;i++)
    {
      signalout->Y[0] += (double) signalin1->Y[i];
    }

  for(i=1;i<size-l-1;i++)
    {
      signalout->Y[i] = (double) signalout->Y[i-1];
      signalout->Y[i] -= (double) signalin1->Y[i-1];
      signalout->Y[i] += (double) signalin1->Y[i+l-1];
    }

  return YES;
}



double bessil0(double x)
{
  double ax,ans;
  double y;

  if ((ax= (double) fabs(x)) < 3.75) {
    y=x/3.75;
    y*=y;
    ans=1.0+y*((double) 3.5156229+y*((double) 3.0899424+y*((double) 1.2067492
							   +y*( (double) 0.2659732+y*( (double) 0.360768e-1+y* (double) 0.45813e-2)))));
  } else {
    y= (double) 3.75/ax;
    ans=((double) exp(ax)/((double) sqrt(ax)))*(0.39894228+y*(0.1328592e-1
							      +y*(0.225319e-2+y*(-0.157565e-2+y*(0.916281e-2
												 +y*(-0.2057706e-1+y*(0.2635537e-1+y*(-0.1647633e-1
																      +y*0.392377e-2))))))));
  }
  return ans;
}


void C_Increment(char **argv)
{
  SIGNAL signalin1,signalout;
  int l;

  argv = ParseArgv(argv,tSIGNALI,&signalin1,tSIGNAL,&signalout,tINT,&l,0);

  if(Increment(signalin1,signalout,l) != YES)
    Errorf("unexpected error in Incr");
}

void C_bessel0(char **argv)
{

  SIGNAL S1,S2;
  int i,N;
  double d;
  argv= ParseArgv(argv,tSIGNALI,&S1,tINT,&N,tSIGNAL,&S2,0);


  SizeSignal(S2,N,YSIG);
  ZeroSig(S2);
  for(i=0;i<N;i++) {
    d= (double) S1->Y[i];
    S2->Y[i]= (double) bessil0(d);
  }
}

static CProc utilProcs[] = {
  "bess0",C_bessel0,"{{{<signalI> <size> <signalO>} {}}}",
  "increment",C_Increment,"{{{<signalI> <signalO> <l>} {}}}",
  NULL,NULL,NULL
};

CProcTable utilTable = {utilProcs,"util","Some utilities"};

